# -*- coding: utf-8 -*-
import gmsh
import sys

from os import replace
from os.path import splitext


def _remove_entities(gmsh, labels):
    """
    Remove model entities that have one of the given labels in their physical
    groups names.

    Parameters
    ----------

    gmsh :
        gmsh object

    labels : list
        list of labels

    """
    # TODO add test that entities are not in surf or part of other 'keeper'entities
    # get all group names
    grps = gmsh.model.getPhysicalGroups(-1)
    grp_names = [gmsh.model.getPhysicalName(*grp) for grp in grps]

    # get entities that will be removed
    pt_list = []
    line_list = []
    surf_list = []
    for grp, name in zip(grps, grp_names):
        if any([label in name.lower() for label in labels]):
            dim = grp[0]
            entities = gmsh.model.getEntitiesForPhysicalGroup(dim, grp[1])
            if dim == 0:
                pt_list.extend(entities.tolist())
            elif dim == 1:
                line_list.extend(entities.tolist())
            elif dim == 2:
                surf_list.extend(entities.tolist())

    # get lines of surfaces
    for surf in surf_list:
        lines = gmsh.model.getBoundary([(2, surf)])  # TODO check if holes are included
        for line in lines:
            line_list.append(line[1])

    # get points of lines
    for line in line_list:
        pts = gmsh.model.getBoundary([(1, line)])
        for pt in pts:
            pt_list.append(pt[1])

    # get unique list of entities to remove
    line_list = list(set(line_list))
    pt_list = list(set(pt_list))

    # delete unused entities
    for surf in surf_list:
        # gmsh.model.removeEntities((2, surf), recursive=False)
        gmsh.model.geo.remove([(2, surf)], recursive=False)

    for line in line_list:
        # gmsh.model.removeEntities((1, line), recursive=False)
        gmsh.model.geo.remove([(1, line)], recursive=False)

    for pt in pt_list:
        # gmsh.model.removeEntities((0, pt), recursive=False)
        gmsh.model.geo.remove([(0, pt)], recursive=False)

    # synchronize to apply changes to model
    gmsh.model.geo.synchronize()


def _get_names_physical(gmsh, dimtag):
    grp_tags = gmsh.model.getPhysicalGroupsForEntity(*dimtag)
    names = [gmsh.model.getPhysicalName(1, tag) for tag in grp_tags]

    return names


def process_mesh(
    self, file_in, file_out, is_get_lam=True, is_get_magnet=False, is_hole_air=True
):
    """Preprocess the GMSH model, i.e. remove unused parts, rename boundaries, ..."""
    # TODO utilize 'translation' dict

    gmsh.initialize(sys.argv)
    gmsh.open(file_in)
    gmsh.model.geo.removeAllDuplicates()

    # remove unused model parts
    _remove_entities(gmsh, labels=["stator", "w_sta"])

    # get group names
    grps = gmsh.model.getPhysicalGroups(-1)
    grp_names = [gmsh.model.getPhysicalName(*grp) for grp in grps]

    # get lists of some surfaces by name
    magnet_list = []
    for grp, name in zip(grps, grp_names):
        if "magnet" in name.lower():
            entities = gmsh.model.getEntitiesForPhysicalGroup(*grp)
            if grp[0] == 2:
                magnet_list.extend(entities.tolist())

    if True:  # is_get_lam:
        lam_list = []
        for grp, name in zip(grps, grp_names):
            if "rotor_lam" in name.lower():
                entities = gmsh.model.getEntitiesForPhysicalGroup(*grp)
                if grp[0] == 2:
                    lam_list.extend(entities.tolist())

        lam_lines = []
        for lam in lam_list:
            lam_lines.extend(gmsh.model.getBoundary([(2, lam)], oriented=False))
        lam_lines = list(set([lam[1] for lam in lam_lines]))  # unique

        hole_lines = []
        for line in lam_lines:
            names = _get_names_physical(
                gmsh,
                dimtag=[
                    1,
                    line,
                ],
            )
            if any(["h_rotor" in name.lower() for name in names]):
                hole_lines.append(line)

    if is_get_lam and not is_get_magnet:
        ext = "Master"
    else:
        ext = "Slave"

    # setup dict to store physical groups, key: group name, value: list of tags
    groups_dict = {}

    # get lines of magnets for processing their physical groups
    for id, magnet in enumerate(magnet_list):
        lines = gmsh.model.getBoundary([(2, magnet)])
        # store new group names in 'groups_dict' to set it later
        for line in lines:
            names = _get_names_physical(
                gmsh,
                dimtag=[
                    1,
                    abs(line[1]),
                ],
            )
            if not names:
                print(f"Warning: Found magnet line without label - line {line[1]}")

            if is_get_magnet or (is_get_lam and line[1] in lam_lines):
                for name in names:
                    if "Magnet" in name:
                        if (
                            line[1] in lam_lines
                        ):  # only lines with direct contact for now
                            # replace number and add 'Slave'
                            s = name.split("_")
                            s.append(ext)  # add extension
                            s[2] = str(id)  # renumber
                            key = "_".join(s)  # new name
                            if key not in groups_dict.keys():
                                groups_dict[key] = []
                            groups_dict[key].append(line)

            # Test if other lines (with 'Hole' phy. group) and same phy. group ext.
            # ('Left', ...) share the same points to add them as well
            # TODO if not used with direct contact (see above),
            #      but I will keep it for Contact Simulation
            # if is_get_lam:
            #     # get the name of the magnets line
            #     s = None
            #     for name in names:
            #         if "Magnet" in name:
            #             s = name.split('_')
            #     # basic check of magnet line name
            #     if s is not None and len(s) >= 3:
            #         for hline in hole_lines:
            #             if hline != abs(line[1]): # skip if hole line == magnet line
            #                 # test for same extension ('Left', 'Right', ...)
            #                 names = _get_names_physical(gmsh, dimtag=[1, hline])
            #                 if any([s[2] in name for name in names]):
            #                     p1 = [x[1] for x in gmsh.model.getBoundary([line])]
            #                     p2 = [x[1] for x in gmsh.model.getBoundary([(1, hline)])]
            #                     pt = [p for p in p1 if p in p2]
            #                     if pt:
            #                         if len(s) == 3:
            #                             s.append('Master') # add extension
            #                         s[1] = str(id) # renumber
            #                         key = "_".join(s) # new name
            #                         if key not in groups_dict.keys():
            #                             groups_dict[key] = []
            #                         groups_dict[key].append((1, hline))
            #         if not is_hole_air:
            #             pass # TODO implement

        if is_get_magnet:
            # store new magnet body name
            s = "Magnet_" + str(id) + "_Body"
            groups_dict[s] = [
                (2, magnet),
            ]

    if is_get_lam:
        # store new lamination body name
        for id, lam in enumerate(lam_list):
            s = "Lamination_" + str(id) + "_Body"
            if s not in groups_dict:
                groups_dict[s] = []
            groups_dict[s].append((2, lam))

        # store hole if not air
        if not is_hole_air:
            pass  # TODO

    # add symmetry boundaries to keeper dict 'groups_dict'
    if is_get_lam:
        keeper_list = [
            "MASTER_ROTOR_BOUNDARY",
            "SLAVE_ROTOR_BOUNDARY",
            "Rotor_Tangential_Bridge",
            "Rotor_Radial_Bridge",
            "ROTOR_BORE_CURVE",
        ]

        for line in lam_lines:
            names = _get_names_physical(gmsh, dimtag=[1, line])
            for key in keeper_list:
                if any([key in name for name in names]):
                    if key not in groups_dict:
                        groups_dict[key] = []
                    groups_dict[key].append((1, line))

    # update group names
    grps = gmsh.model.getPhysicalGroups(-1)
    grp_names = [gmsh.model.getPhysicalName(*grp) for grp in grps]

    # delete unused surfaces
    del_list = ["shaft", "h_rotor"]
    if not is_get_magnet:
        del_list.append("magnet")

    if not is_get_lam:
        del_list.append("rotor_lam")

    for grp, name in zip(grps, grp_names):
        if any([n in name.lower() for n in del_list]):
            entities = gmsh.model.getEntitiesForPhysicalGroup(*grp).tolist()
            for entity in entities:
                if grp[0] == 2:
                    gmsh.model.geo.remove([(2, entity)], recursive=False)

    # set new physical group names after removing all 'old' physical groups
    gmsh.model.removePhysicalGroups(dimTags=[])
    for name in grp_names:
        gmsh.model.removePhysicalName(name)

    for key, values in groups_dict.items():
        # lines
        tags = [abs(dimtag[1]) for dimtag in values if dimtag[0] == 1]
        tags = list(set(tags))
        if tags:
            print(f"Add physical group {key} with lines {tags}")
            pg = gmsh.model.addPhysicalGroup(1, tags, tag=-1)
            gmsh.model.setPhysicalName(1, pg, key)
        # surfaces
        tags = [abs(dimtag[1]) for dimtag in values if dimtag[0] == 2]
        tags = list(set(tags))
        if tags:
            print(f"Add physical group {key} with surface {tags}")
            pg = gmsh.model.addPhysicalGroup(2, tags, tag=-1)
            gmsh.model.setPhysicalName(2, pg, key)

    # cleanup
    # get all entities
    gmsh.model.geo.synchronize()
    entities_all = gmsh.model.getEntities(dim=-1)
    surf_list = gmsh.model.getEntities(dim=2)
    print(surf_list)

    # get lines of surfaces
    line_list = []
    for surf in surf_list:
        line_list.extend(gmsh.model.getBoundary([surf]))
    line_list = [(line[0], abs(line[1])) for line in line_list]

    # remove all lines that are not part of the surfaces
    for entity in entities_all:
        if (entity[0], abs(entity[1])) not in line_list and entity[0] == 1:
            gmsh.model.geo.remove([entity], recursive=False)

    # entities_all = gmsh.model.getEntities(dim=-1)

    # remove unknown/unused physical groups
    # TODO

    # save
    gmsh.model.geo.synchronize()

    # gmsh.model.geo.mesh.setTransfiniteSurface(tag)
    for surf in surf_list:
        gmsh.model.mesh.setRecombine(2, surf[1])

    # gmsh.option.setNumber("Mesh.RecombinationAlgorithm", 1)

    gmsh.model.mesh.generate(2)
    # gmsh.model.mesh.recombine()
    gmsh.model.mesh.refine()
    # gmsh.model.mesh.refine()
    # gmsh.model.mesh.recombine()
    # gmsh.model.mesh.refine()

    # save mesh or geo file depending on file extension
    filename, file_extension = splitext(file_out)

    if file_extension == ".geo":
        gmsh.write(filename + ".geo_unrolled")
        replace(filename + ".geo_unrolled", filename + file_extension)
    else:
        gmsh.model.mesh.generate(2)
        gmsh.write(file_out)

    # gmsh.fltk.run()      # Uncomment to launch Gmsh GUI

    # update group names once again
    grps = gmsh.model.getPhysicalGroups(-1)
    grp_names = [gmsh.model.getPhysicalName(*grp) for grp in grps]

    gmsh.finalize()

    return gmsh, grps, grp_names



# -*- coding: utf-8 -*-

from numpy import array


def comp_height(self, Ndisc=200):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : Slot
        A Slot object
    Ndisc : int
        Number of point to discretize the lines

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    Rbo = self.get_Rbo()

    surf = self.get_surface()

    point_list = surf.discretize(Ndisc)
    point_list = array(point_list)
    if self.is_outwards():
        return max(abs(point_list)) - Rbo
    else:
        return Rbo - min(abs(point_list))



# -*- coding: utf-8 -*-
from numpy import dot


def is_inside(self, vertice, point, normal_t=None):
    """Check if a point is inside the cell defined by the vertices.

    Parameters
    ----------
    self : RefTriangle3
        an RefTriangle3 object
    vertice : ndarray
        vertices of the cell
    point : ndarray
        coordinates of the checked point
    normal_t : ndarray
        (optional) cell normal vector

    Returns
    -------
    is_inside: bool
        true if the point is inside the cell

    """
    # TODO
    return None



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Simulation/Simulation.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Simulation/Simulation
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from ._frozen import FrozenClass

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Simulation.Simulation.run import run
except ImportError as error:
    run = error

try:
    from ..Methods.Simulation.Simulation.init_logger import init_logger
except ImportError as error:
    init_logger = error


from ._check import InitUnKnowClassError
from .Machine import Machine
from .Input import Input
from .VarSimu import VarSimu
from .Post import Post


class Simulation(FrozenClass):
    """Abstract class for the simulation"""

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Simulation.Simulation.run
    if isinstance(run, ImportError):
        run = property(
            fget=lambda x: raise_(
                ImportError("Can't use Simulation method run: " + str(run))
            )
        )
    else:
        run = run
    # cf Methods.Simulation.Simulation.init_logger
    if isinstance(init_logger, ImportError):
        init_logger = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use Simulation method init_logger: " + str(init_logger)
                )
            )
        )
    else:
        init_logger = init_logger
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        name="",
        desc="",
        machine=-1,
        input=-1,
        logger_name="Pyleecan.Simulation",
        var_simu=None,
        postproc_list=-1,
        index=None,
        path_result=None,
        layer=None,
        layer_log_warn=None,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "name" in list(init_dict.keys()):
                name = init_dict["name"]
            if "desc" in list(init_dict.keys()):
                desc = init_dict["desc"]
            if "machine" in list(init_dict.keys()):
                machine = init_dict["machine"]
            if "input" in list(init_dict.keys()):
                input = init_dict["input"]
            if "logger_name" in list(init_dict.keys()):
                logger_name = init_dict["logger_name"]
            if "var_simu" in list(init_dict.keys()):
                var_simu = init_dict["var_simu"]
            if "postproc_list" in list(init_dict.keys()):
                postproc_list = init_dict["postproc_list"]
            if "index" in list(init_dict.keys()):
                index = init_dict["index"]
            if "path_result" in list(init_dict.keys()):
                path_result = init_dict["path_result"]
            if "layer" in list(init_dict.keys()):
                layer = init_dict["layer"]
            if "layer_log_warn" in list(init_dict.keys()):
                layer_log_warn = init_dict["layer_log_warn"]
        # Set the properties (value check and convertion are done in setter)
        self.parent = None
        self.name = name
        self.desc = desc
        self.machine = machine
        self.input = input
        self.logger_name = logger_name
        self.var_simu = var_simu
        self.postproc_list = postproc_list
        self.index = index
        self.path_result = path_result
        self.layer = layer
        self.layer_log_warn = layer_log_warn

        # The class is frozen, for now it's impossible to add new properties
        self._freeze()

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        Simulation_str = ""
        if self.parent is None:
            Simulation_str += "parent = None " + linesep
        else:
            Simulation_str += "parent = " + str(type(self.parent)) + " object" + linesep
        Simulation_str += 'name = "' + str(self.name) + '"' + linesep
        Simulation_str += 'desc = "' + str(self.desc) + '"' + linesep
        if self.machine is not None:
            tmp = self.machine.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simulation_str += "machine = " + tmp
        else:
            Simulation_str += "machine = None" + linesep + linesep
        if self.input is not None:
            tmp = self.input.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simulation_str += "input = " + tmp
        else:
            Simulation_str += "input = None" + linesep + linesep
        Simulation_str += 'logger_name = "' + str(self.logger_name) + '"' + linesep
        if self.var_simu is not None:
            tmp = self.var_simu.__str__().replace(linesep, linesep + "\t").rstrip("\t")
            Simulation_str += "var_simu = " + tmp
        else:
            Simulation_str += "var_simu = None" + linesep + linesep
        if len(self.postproc_list) == 0:
            Simulation_str += "postproc_list = []" + linesep
        for ii in range(len(self.postproc_list)):
            tmp = (
                self.postproc_list[ii].__str__().replace(linesep, linesep + "\t")
                + linesep
            )
            Simulation_str += (
                "postproc_list[" + str(ii) + "] =" + tmp + linesep + linesep
            )
        Simulation_str += "index = " + str(self.index) + linesep
        Simulation_str += 'path_result = "' + str(self.path_result) + '"' + linesep
        Simulation_str += "layer = " + str(self.layer) + linesep
        Simulation_str += "layer_log_warn = " + str(self.layer_log_warn) + linesep
        return Simulation_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False
        if other.name != self.name:
            return False
        if other.desc != self.desc:
            return False
        if other.machine != self.machine:
            return False
        if other.input != self.input:
            return False
        if other.logger_name != self.logger_name:
            return False
        if other.var_simu != self.var_simu:
            return False
        if other.postproc_list != self.postproc_list:
            return False
        if other.index != self.index:
            return False
        if other.path_result != self.path_result:
            return False
        if other.layer != self.layer:
            return False
        if other.layer_log_warn != self.layer_log_warn:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()
        if other._name != self._name:
            diff_list.append(name + ".name")
        if other._desc != self._desc:
            diff_list.append(name + ".desc")
        if (other.machine is None and self.machine is not None) or (
            other.machine is not None and self.machine is None
        ):
            diff_list.append(name + ".machine None mismatch")
        elif self.machine is not None:
            diff_list.extend(
                self.machine.compare(other.machine, name=name + ".machine")
            )
        if (other.input is None and self.input is not None) or (
            other.input is not None and self.input is None
        ):
            diff_list.append(name + ".input None mismatch")
        elif self.input is not None:
            diff_list.extend(self.input.compare(other.input, name=name + ".input"))
        if other._logger_name != self._logger_name:
            diff_list.append(name + ".logger_name")
        if (other.var_simu is None and self.var_simu is not None) or (
            other.var_simu is not None and self.var_simu is None
        ):
            diff_list.append(name + ".var_simu None mismatch")
        elif self.var_simu is not None:
            diff_list.extend(
                self.var_simu.compare(other.var_simu, name=name + ".var_simu")
            )
        if (other.postproc_list is None and self.postproc_list is not None) or (
            other.postproc_list is not None and self.postproc_list is None
        ):
            diff_list.append(name + ".postproc_list None mismatch")
        elif self.postproc_list is None:
            pass
        elif len(other.postproc_list) != len(self.postproc_list):
            diff_list.append("len(" + name + ".postproc_list)")
        else:
            for ii in range(len(other.postproc_list)):
                diff_list.extend(
                    self.postproc_list[ii].compare(
                        other.postproc_list[ii],
                        name=name + ".postproc_list[" + str(ii) + "]",
                    )
                )
        if other._index != self._index:
            diff_list.append(name + ".index")
        if other._path_result != self._path_result:
            diff_list.append(name + ".path_result")
        if other._layer != self._layer:
            diff_list.append(name + ".layer")
        if other._layer_log_warn != self._layer_log_warn:
            diff_list.append(name + ".layer_log_warn")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object
        S += getsizeof(self.name)
        S += getsizeof(self.desc)
        S += getsizeof(self.machine)
        S += getsizeof(self.input)
        S += getsizeof(self.logger_name)
        S += getsizeof(self.var_simu)
        if self.postproc_list is not None:
            for value in self.postproc_list:
                S += getsizeof(value)
        S += getsizeof(self.index)
        S += getsizeof(self.path_result)
        S += getsizeof(self.layer)
        S += getsizeof(self.layer_log_warn)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        Simulation_dict = dict()
        Simulation_dict["name"] = self.name
        Simulation_dict["desc"] = self.desc
        if self.machine is None:
            Simulation_dict["machine"] = None
        else:
            Simulation_dict["machine"] = self.machine.as_dict(**kwargs)
        if self.input is None:
            Simulation_dict["input"] = None
        else:
            Simulation_dict["input"] = self.input.as_dict(**kwargs)
        Simulation_dict["logger_name"] = self.logger_name
        if self.var_simu is None:
            Simulation_dict["var_simu"] = None
        else:
            Simulation_dict["var_simu"] = self.var_simu.as_dict(**kwargs)
        if self.postproc_list is None:
            Simulation_dict["postproc_list"] = None
        else:
            Simulation_dict["postproc_list"] = list()
            for obj in self.postproc_list:
                if obj is not None:
                    Simulation_dict["postproc_list"].append(obj.as_dict(**kwargs))
                else:
                    Simulation_dict["postproc_list"].append(None)
        Simulation_dict["index"] = self.index
        Simulation_dict["path_result"] = self.path_result
        Simulation_dict["layer"] = self.layer
        Simulation_dict["layer_log_warn"] = self.layer_log_warn
        # The class name is added to the dict for deserialisation purpose
        Simulation_dict["__class__"] = "Simulation"
        return Simulation_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.name = None
        self.desc = None
        if self.machine is not None:
            self.machine._set_None()
        if self.input is not None:
            self.input._set_None()
        self.logger_name = None
        if self.var_simu is not None:
            self.var_simu._set_None()
        self.postproc_list = None
        self.index = None
        self.path_result = None
        self.layer = None
        self.layer_log_warn = None

    def _get_name(self):
        """getter of name"""
        return self._name

    def _set_name(self, value):
        """setter of name"""
        check_var("name", value, "str")
        self._name = value

    name = property(
        fget=_get_name,
        fset=_set_name,
        doc=u"""Name of the simulation

        :Type: str
        """,
    )

    def _get_desc(self):
        """getter of desc"""
        return self._desc

    def _set_desc(self, value):
        """setter of desc"""
        check_var("desc", value, "str")
        self._desc = value

    desc = property(
        fget=_get_desc,
        fset=_set_desc,
        doc=u"""Simulation description

        :Type: str
        """,
    )

    def _get_machine(self):
        """getter of machine"""
        return self._machine

    def _set_machine(self, value):
        """setter of machine"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "machine"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Machine()
        check_var("machine", value, "Machine")
        self._machine = value

        if self._machine is not None:
            self._machine.parent = self

    machine = property(
        fget=_get_machine,
        fset=_set_machine,
        doc=u"""Machine to simulate

        :Type: Machine
        """,
    )

    def _get_input(self):
        """getter of input"""
        return self._input

    def _set_input(self, value):
        """setter of input"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "input"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = Input()
        check_var("input", value, "Input")
        self._input = value

        if self._input is not None:
            self._input.parent = self

    input = property(
        fget=_get_input,
        fset=_set_input,
        doc=u"""Input of the simulation

        :Type: Input
        """,
    )

    def _get_logger_name(self):
        """getter of logger_name"""
        return self._logger_name

    def _set_logger_name(self, value):
        """setter of logger_name"""
        check_var("logger_name", value, "str")
        self._logger_name = value

    logger_name = property(
        fget=_get_logger_name,
        fset=_set_logger_name,
        doc=u"""Name of the logger to use

        :Type: str
        """,
    )

    def _get_var_simu(self):
        """getter of var_simu"""
        return self._var_simu

    def _set_var_simu(self, value):
        """setter of var_simu"""
        if isinstance(value, str):  # Load from file
            value = load_init_dict(value)[1]
        if isinstance(value, dict) and "__class__" in value:
            class_obj = import_class(
                "pyleecan.Classes", value.get("__class__"), "var_simu"
            )
            value = class_obj(init_dict=value)
        elif type(value) is int and value == -1:  # Default constructor
            value = VarSimu()
        check_var("var_simu", value, "VarSimu")
        self._var_simu = value

        if self._var_simu is not None:
            self._var_simu.parent = self

    var_simu = property(
        fget=_get_var_simu,
        fset=_set_var_simu,
        doc=u"""Multi-simulation definition

        :Type: VarSimu
        """,
    )

    def _get_postproc_list(self):
        """getter of postproc_list"""
        if self._postproc_list is not None:
            for obj in self._postproc_list:
                if obj is not None:
                    obj.parent = self
        return self._postproc_list

    def _set_postproc_list(self, value):
        """setter of postproc_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "postproc_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("postproc_list", value, "[Post]")
        self._postproc_list = value

    postproc_list = property(
        fget=_get_postproc_list,
        fset=_set_postproc_list,
        doc=u"""List of postprocessings to run on Output after the simulation

        :Type: [Post]
        """,
    )

    def _get_index(self):
        """getter of index"""
        return self._index

    def _set_index(self, value):
        """setter of index"""
        check_var("index", value, "int", Vmin=0)
        self._index = value

    index = property(
        fget=_get_index,
        fset=_set_index,
        doc=u"""Index of the simulation (if part of a multi-simulation)

        :Type: int
        :min: 0
        """,
    )

    def _get_path_result(self):
        """getter of path_result"""
        return self._path_result

    def _set_path_result(self, value):
        """setter of path_result"""
        check_var("path_result", value, "str")
        self._path_result = value

    path_result = property(
        fget=_get_path_result,
        fset=_set_path_result,
        doc=u"""Path to the Result folder to use (None to use default one)

        :Type: str
        """,
    )

    def _get_layer(self):
        """getter of layer"""
        return self._layer

    def _set_layer(self, value):
        """setter of layer"""
        check_var("layer", value, "int", Vmin=0)
        self._layer = value

    layer = property(
        fget=_get_layer,
        fset=_set_layer,
        doc=u"""Layer of the simulation in a multi-simulation (0 is top simulation)

        :Type: int
        :min: 0
        """,
    )

    def _get_layer_log_warn(self):
        """getter of layer_log_warn"""
        return self._layer_log_warn

    def _set_layer_log_warn(self, value):
        """setter of layer_log_warn"""
        check_var("layer_log_warn", value, "int", Vmin=0)
        self._layer_log_warn = value

    layer_log_warn = property(
        fget=_get_layer_log_warn,
        fset=_set_layer_log_warn,
        doc=u"""Enable to set the log console_handler to warning starting from a particular layer. layer_log_warn=2 => layer 0 and 1 info, layer 2 warning

        :Type: int
        :min: 0
        """,
    )



# -*- coding: utf-8 -*-


def get_indices_limited(
    meshsolution,
    label=None,
    group_names=None,
    max_value=float("Inf"),
    min_value=float("-Inf"),
):
    """Get the indices of the specified groups of a MeshSolution,
    with the field 'label' values magnitude must be within the given limits.

    Parameters
    ----------
    meshsolution : MeshSolution
        a MeshSolution object

    label : str
        label of a MeshSolution solution field

    group_names : list of str
        list of the groups names or string of a single group

    max_value : float
        max. field magnitude (over time)

    min_value : float
        min. field magnitude (over time)

    Return
    ------
    area : float
        surface area of the specified groups and indices

    """
    grp = meshsolution.get_group(group_names=group_names)
    msh = grp.get_mesh()

    indices = msh.cell["triangle"].indice

    field = grp.get_field(label=label)

    # TODO check field size and axes
    field_mag = (field[:, :, 0] ** 2 + field[:, :, 1] ** 2) ** (1 / 2)
    field_max = field_mag.max(axis=0)
    field_min = field_mag.min(axis=0)

    idx_max = field_max >= max_value
    idx_min = field_min <= min_value

    idx = idx_max + idx_min

    if any(idx):
        inds = indices[idx]
    else:
        inds = []

    return inds



# -*- coding: utf-8 -*-

from matplotlib.patches import Patch, Polygon
from matplotlib.pyplot import axis, legend
from numpy import array, exp, pi

from ....definitions import config_dict
from ....Functions.init_fig import init_fig
from ....Functions.Winding.gen_phase_list import gen_name

PHASE_COLORS = config_dict["PLOT"]["COLOR_DICT"]["PHASE_COLORS"]


def plot_active(
    self,
    wind_mat=None,
    fig=None,
    is_bar=False,
    is_show_fig=True,
    enforced_default_color=None,
    alpha=0,
    delta=0,
):
    """Plot the active area of the lamination according to the wind_mat

    Parameters
    ----------
    self : Slot
        A Slot object
    wind_mat : numpy.ndarray
        A matrix [Nrad,Ntan,Zs,qs] representing the active (Default value = None)
    fig :
        if None, open a new fig and plot, else add to the current
        one (Default value = None)
    is_bar : bool
        To adapt the legend text for squirrel cage bar (Default value = False)
    is_show_fig : bool
        To call show at the end of the method
    enforced_default_color : str
        If not None enforce the active color (when wind_mat is None)
    alpha : float
        Angle for rotation (Default value = 0) [rad]
    delta : Complex
        complex for translation (Default value = 0)
    Returns
    -------
    None
    """

    if enforced_default_color is None:
        enforced_default_color = PHASE_COLORS[0]

    if wind_mat is None:  # Default : Only one zone monocolor
        Nrad, Ntan, qs = 1, 1, 1
        Zs = self.Zs
    else:
        (Nrad, Ntan, Zs, qs) = wind_mat.shape
    qs_name = gen_name(qs)

    surf_list = self.build_geometry_active(Nrad, Ntan, alpha=alpha, delta=delta)

    patches = list()
    for ii in range(len(surf_list)):
        # Compute the coordinate for one zone
        point_list = list()
        for curve in surf_list[ii].get_lines():
            point_list.extend(curve.discretize().tolist())
        point_list = array(point_list)

        for jj in range(Zs):
            if wind_mat is None or len(surf_list) != Ntan * Nrad:
                x, y = point_list.real, point_list.imag
                patches.append(Polygon(list(zip(x, y)), color=enforced_default_color))
            else:
                # print "Nrad, Ntan, Zs : "+str((ii%Nrad,ii/Nrad,jj))
                color = get_color(wind_mat, ii % Nrad, ii // Nrad, jj)
                x, y = point_list.real, point_list.imag
                patches.append(Polygon(list(zip(x, y)), color=color))
            point_list = point_list * exp(1j * (2 * pi) / self.Zs)

    # Display the result
    (fig, axes, patch_leg, label_leg) = init_fig(fig)
    axes.set_xlabel("(m)")
    axes.set_ylabel("(m)")
    axes.set_title("Winding Pattern")

    # Add the magnet to the fig
    for patch in patches:
        axes.add_patch(patch)

    # Axis Setup
    axis("equal")
    Rbo = self.get_Rbo()

    Lim = Rbo * 1.2
    axes.set_xlim(-Lim, Lim)
    axes.set_ylim(-Lim, Lim)

    # Legend setup
    if wind_mat is None or len(surf_list) != Ntan * Nrad:
        # No winding matrix => Only one zone
        if not is_bar and not ("Winding" in label_leg):
            # Avoid adding twice the same label
            patch_leg.append(Patch(color=PHASE_COLORS[0]))
            label_leg.append("Winding")
        elif is_bar and not ("Rotor bar" in label_leg):
            # Avoid adding twice the same label
            patch_leg.append(Patch(color=PHASE_COLORS[0]))
            label_leg.append("Rotor bar")
    else:  # Add every phase to the legend
        for ii in range(qs):
            if not ("Phase " + qs_name[ii] in label_leg):
                # Avoid adding twice the same label
                index = ii % len(PHASE_COLORS)
                patch_leg.append(Patch(color=PHASE_COLORS[index]))
                label_leg.append("Phase " + qs_name[ii])

    legend(patch_leg, label_leg)
    if is_show_fig:
        fig.show()


def get_color(wind_mat, Nrad, Ntan, Zs):
    """Return the color (corresponding phase) for the zone (Nrad,Ntan,Zs)

    Parameters
    ----------
    wind_mat :
        A matrix [Nrad,Ntan,Zs,qs] representing the winding
    Nrad :
        Zone radial coordinate
    Ntan :
        Zone tagential coordinate
    Zs :
        Zone slot number coordinate

    Returns
    -------
    str
        color: Color of the zone

    """
    A = wind_mat[Nrad, Ntan, Zs, :]
    for zz in range(len(A)):
        if A[zz] != 0:
            return PHASE_COLORS[zz]
    return "w"  # If all the phase are at 0 : the zone is empty => white



# -*- coding: utf-8 -*-
"""File generated according to PWSlot14/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWSlot.PWSlot14.Ui_PWSlot14 import Ui_PWSlot14


class Gen_PWSlot14(Ui_PWSlot14):
    def setupUi(self, PWSlot14):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PWSlot14.setupUi(self, PWSlot14)
        # Setup of in_W0
        txt = self.tr(u"""Slot isthmus width.""")
        self.in_W0.setWhatsThis(txt)
        self.in_W0.setToolTip(txt)

        # Setup of lf_W0
        self.lf_W0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus width.""")
        self.lf_W0.setWhatsThis(txt)
        self.lf_W0.setToolTip(txt)

        # Setup of in_W3
        txt = self.tr(u"""Tooth width""")
        self.in_W3.setWhatsThis(txt)
        self.in_W3.setToolTip(txt)

        # Setup of lf_W3
        self.lf_W3.validator().setBottom(0)
        txt = self.tr(u"""Tooth width""")
        self.lf_W3.setWhatsThis(txt)
        self.lf_W3.setToolTip(txt)

        # Setup of in_H0
        txt = self.tr(u"""Slot isthmus height.""")
        self.in_H0.setWhatsThis(txt)
        self.in_H0.setToolTip(txt)

        # Setup of lf_H0
        self.lf_H0.validator().setBottom(0)
        txt = self.tr(u"""Slot isthmus height.""")
        self.lf_H0.setWhatsThis(txt)
        self.lf_H0.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Slot intermediate height.""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Slot intermediate height.""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)

        # Setup of in_H3
        txt = self.tr(u"""Tooth height""")
        self.in_H3.setWhatsThis(txt)
        self.in_H3.setToolTip(txt)

        # Setup of lf_H3
        self.lf_H3.validator().setBottom(0)
        txt = self.tr(u"""Tooth height""")
        self.lf_H3.setWhatsThis(txt)
        self.lf_H3.setToolTip(txt)



# -*- coding: utf-8 -*-

from PySide2.QtCore import Signal
from PySide2.QtWidgets import QWidget

from .......GUI import gui_option
from .......GUI.Dialog.DMachineSetup.SLamParam.DAVDuct.PVentTrap.Gen_PVentTrap import (
    Gen_PVentTrap,
)


class PVentTrap(Gen_PVentTrap, QWidget):
    """Page to setup the Ventilation Trap"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()

    def __init__(self, lam=None, vent=None):
        """Initialize the widget according the current lamination

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap widget
        lam : Lamination
            current lamination to edit
        vent : VentTrap
            current ventilation to edit
        """

        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Set FloatEdit unit
        self.lf_H0.unit = "m"
        self.lf_D0.unit = "m"
        self.lf_W1.unit = "m"
        self.lf_W2.unit = "m"

        self.lam = lam
        self.vent = vent

        # Fill the fields with the machine values (if they're filled)
        if self.vent.Zh is None:
            self.vent.Zh = 8
        self.si_Zh.setValue(self.vent.Zh)
        self.lf_H0.setValue(self.vent.H0)
        self.lf_D0.setValue(self.vent.D0)
        self.lf_W1.setValue(self.vent.W1)
        self.lf_W2.setValue(self.vent.W2)
        self.lf_Alpha0.setValue(self.vent.Alpha0)

        # Display the main output of the vent
        self.w_out.comp_output()

        # Set unit name (m ou mm)
        wid_list = [self.unit_H0, self.unit_D0, self.unit_W1, self.unit_W1]
        for wid in wid_list:
            wid.setText("[" + gui_option.unit.get_m_name() + "]")

        # Connect the signal
        self.si_Zh.editingFinished.connect(self.set_Zh)
        self.lf_H0.editingFinished.connect(self.set_H0)
        self.lf_D0.editingFinished.connect(self.set_D0)
        self.lf_W1.editingFinished.connect(self.set_W1)
        self.lf_W2.editingFinished.connect(self.set_W2)
        self.lf_Alpha0.editingFinished.connect(self.set_Alpha0)

    def set_Zh(self):
        """Signal to update the value of Zh according to the line edit

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object
        """
        self.vent.Zh = self.si_Zh.value()
        self.w_out.comp_output()

    def set_H0(self):
        """Signal to update the value of H0 according to the line edit

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object
        """
        self.vent.H0 = self.lf_H0.value()
        self.w_out.comp_output()

    def set_D0(self):
        """Signal to update the value of D0 according to the line edit

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object
        """
        self.vent.D0 = self.lf_D0.value()
        self.w_out.comp_output()

    def set_W1(self):
        """Signal to update the value of W1 according to the line edit

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object
        """
        self.vent.W1 = self.lf_W1.value()
        self.w_out.comp_output()

    def set_W2(self):
        """Signal to update the value of W2 according to the line edit

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object
        """
        self.vent.W2 = self.lf_W2.value()
        self.w_out.comp_output()

    def set_Alpha0(self):
        """Signal to update the value of Alpha0 according to the line edit

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object
        """
        self.vent.Alpha0 = self.lf_Alpha0.value()
        self.w_out.comp_output()

    def check(self):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        self : PVentTrap
            A PVentTrap object

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        # Check that everything is set
        if self.vent.Zh is None:
            return self.tr("You must set Zh !")
        elif self.vent.H0 is None:
            return self.tr("You must set H0 !")
        elif self.vent.D0 is None:
            return self.tr("You must set D0 !")
        elif self.vent.W1 is None:
            return self.tr("You must set W1 !")
        elif self.vent.W2 is None:
            return self.tr("You must set W2 !")
        elif self.vent.Alpha0 is None:
            return self.tr("You must set Alpha0 !")
        return None



# -*- coding: utf-8 -*-

from numpy import pi
from PySide2.QtCore import Signal
from PySide2.QtGui import QPixmap
from PySide2.QtWidgets import QWidget, QDialog
from PySide2.QtWidgets import QMessageBox
from PySide2.QtCore import Qt
from ......Functions.load import load
from ......Classes.SlotUD import SlotUD
from ......GUI import gui_option
from ......GUI.Dialog.DMachineSetup.SWSlot.PWSlotUD.Ui_PWSlotUD import Ui_PWSlotUD
from ......GUI.Dialog.DMatLib.MatLib import MatLib
from ......Methods.Slot.Slot import SlotCheckError
from ......GUI.Dxf.DXF_Slot import DXF_Slot
from ......GUI.Dialog.DMatLib.WMatSelect.WMatSelect import WMatSelect


class PWSlotUD(Ui_PWSlotUD, QWidget):
    """Page to set the slot from DXF"""

    # Signal to DMachineSetup to know that the save popup is needed
    saveNeeded = Signal()
    # Information for WslotMag
    slot_name = "Import from DXF"
    slot_type = SlotUD

    def __init__(self, lamination=None):
        """Initialize the widget according to lamination

        Parameters
        ----------
        self : PWSlotUD
            A PWSlotUD widget
        lamination : Lamination
            current lamination to edit
        """
        # Build the interface according to the .ui file
        QWidget.__init__(self)
        self.setupUi(self)

        # Set properties
        self.lamination = lamination
        self.slot = lamination.slot
        self.u = gui_option.unit

        # Setup Path selector for Json files
        self.w_path_json.obj = None
        self.w_path_json.param_name = None
        self.w_path_json.verbose_name = "Load from json"
        self.w_path_json.extension = "JSON file (*.json)"
        self.w_path_json.update()

        # Update the GUI according to the current slot
        self.update_graph()
        self.w_out.comp_output()

        # Connect the signals
        self.b_dxf.clicked.connect(self.open_DXF_Slot)
        self.w_path_json.pathChanged.connect(self.load_slot)

    def update_graph(self):
        """Plot the lamination with/without the slot"""
        # Use a copy to avoid changing the main object
        lam = self.lamination.copy()
        try:
            self.slot.check()
            lam.slot = self.slot
        except SlotCheckError:
            # Plot only the lamination
            lam.slot = None
        # Plot the lamination in the viewer fig
        lam.plot(fig=self.w_viewer.fig, is_show_fig=False)

        # Update the Graph
        self.w_viewer.axes.set_axis_off()
        self.w_viewer.axes.axis("equal")
        if self.w_viewer.axes.get_legend() is not None:
            self.w_viewer.axes.get_legend().remove()
        self.w_viewer.draw()

    def load_slot(self):
        """Load the selected json file and display the slot"""
        # Check that the json file is correct
        try:
            slot = load(self.w_path_json.get_path())
        except Exception as e:
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr("Error when loading file:\n" + str(e)),
            )
            return
        # Check that the json file contain a SlotUD
        if not isinstance(slot, SlotUD):
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr(
                    "The choosen file is not a SlotUD file (" + str(type(slot)) + ")"
                ),
            )
            return

        # Update the slot object
        Zs = self.slot.Zs
        parent = self.slot.parent
        self.slot.__init__(init_dict=slot.as_dict())  # keep pointer
        self.slot.Zs = Zs
        self.slot.parent = parent

        # Update the new GUI according to the slot
        self.update_graph()
        self.w_out.comp_output()

    def open_DXF_Slot(self):
        """Open the GUI to define the SlotUD"""
        # Init GUI with lamination parameters
        self.dxf_gui = DXF_Slot(Zs=self.slot.Zs, lam=self.lamination)
        self.dxf_gui.setWindowFlags(Qt.Window)  # To maximize the GUI
        self.dxf_gui.show()
        # Update the slot when saving
        self.dxf_gui.accepted.connect(self.set_dxf_path)

    def set_dxf_path(self):
        """Update the slot according to the file defined by DXF_Slot"""
        # Get the saving path from DXF_Slot
        self.w_path_json.set_path_txt(self.dxf_gui.save_path)
        # Update slot and GUI
        self.load_slot()
        self.dxf_gui = None

    @staticmethod
    def check(lam):
        """Check that the current machine have all the needed field set

        Parameters
        ----------
        lam: LamSlotWind
            Lamination to check

        Returns
        -------
        error: str
            Error message (return None if no error)
        """

        # Constraints
        try:
            lam.slot.check()
        except SlotCheckError as error:
            return str(error)

        # Output
        try:
            yoke_height = lam.comp_height_yoke()
        except Exception as error:
            return translate("Unable to compute yoke height:", "PWSlotUD") + str(error)

        if yoke_height <= 0:
            return translate(
                "The slot height is greater than the lamination !", "PWSlotUD"
            )

    def emit_save(self):
        """Send a saveNeeded signal to the DMachineSetup"""
        self.saveNeeded.emit()



from ....Methods import ParentMissingError


def get_name_lam(self):
    """Return the name of the parent lamination

    Parameters
    ----------
    self : Slot
        A Slot object

    Returns
    -------
    name: string
        The name of the parent lamination

    """

    if self.parent is not None:
        if self.get_is_stator():
            name = "Stator"
        else:
            name = "Rotor"
        return name
    else:
        raise ParentMissingError("Error: The slot is not inside a Lamination")



# -*- coding: utf-8 -*-


def get_data(self):
    """Method to return the Elmer simulation results

    Parameter
    ---------


    Return
    ------
    data : list
        list of data

    """
    # check if data are loaded
    if not self.data:
        self.load_data()

    # output data as SciDataTool data
    # TODO



# -*- coding: utf-8 -*-

# File generated according to SWindParam.ui
# WARNING! All changes made in this file will be lost!
## WARNING! All changes made in this file will be lost when recompiling UI file!
################################################################################

from PySide2.QtCore import *
from PySide2.QtGui import *
from PySide2.QtWidgets import *

from pyleecan.GUI.Resources import pyleecan_rc


class Ui_SWindParam(object):
    def setupUi(self, SWindParam):
        if not SWindParam.objectName():
            SWindParam.setObjectName(u"SWindParam")
        SWindParam.resize(650, 550)
        SWindParam.setMinimumSize(QSize(650, 550))
        self.verticalLayout_3 = QVBoxLayout(SWindParam)
        self.verticalLayout_3.setObjectName(u"verticalLayout_3")
        self.horizontalLayout_3 = QHBoxLayout()
        self.horizontalLayout_3.setObjectName(u"horizontalLayout_3")
        self.img_wind_geo = QLabel(SWindParam)
        self.img_wind_geo.setObjectName(u"img_wind_geo")
        self.img_wind_geo.setMinimumSize(QSize(300, 300))
        self.img_wind_geo.setPixmap(
            QPixmap(u":/images/images/MachineSetup/WindParam/Winding param.PNG")
        )
        self.img_wind_geo.setScaledContents(True)

        self.horizontalLayout_3.addWidget(self.img_wind_geo)

        self.widget = QWidget(SWindParam)
        self.widget.setObjectName(u"widget")
        self.widget.setMinimumSize(QSize(250, 0))
        self.widget.setMaximumSize(QSize(250, 16777215))
        self.verticalLayout_2 = QVBoxLayout(self.widget)
        self.verticalLayout_2.setObjectName(u"verticalLayout_2")
        self.horizontalLayout = QHBoxLayout()
        self.horizontalLayout.setObjectName(u"horizontalLayout")
        self.in_Npcpp = QLabel(self.widget)
        self.in_Npcpp.setObjectName(u"in_Npcpp")
        self.in_Npcpp.setMinimumSize(QSize(40, 0))

        self.horizontalLayout.addWidget(self.in_Npcpp)

        self.si_Npcpp = QSpinBox(self.widget)
        self.si_Npcpp.setObjectName(u"si_Npcpp")
        sizePolicy = QSizePolicy(QSizePolicy.Expanding, QSizePolicy.Fixed)
        sizePolicy.setHorizontalStretch(0)
        sizePolicy.setVerticalStretch(0)
        sizePolicy.setHeightForWidth(self.si_Npcpp.sizePolicy().hasHeightForWidth())
        self.si_Npcpp.setSizePolicy(sizePolicy)
        self.si_Npcpp.setMinimumSize(QSize(100, 0))
        self.si_Npcpp.setMaximumSize(QSize(100, 16777215))
        self.si_Npcpp.setValue(99)

        self.horizontalLayout.addWidget(self.si_Npcpp)

        self.verticalLayout_2.addLayout(self.horizontalLayout)

        self.horizontalLayout_2 = QHBoxLayout()
        self.horizontalLayout_2.setObjectName(u"horizontalLayout_2")
        self.in_Ntcoil = QLabel(self.widget)
        self.in_Ntcoil.setObjectName(u"in_Ntcoil")
        self.in_Ntcoil.setMinimumSize(QSize(40, 0))

        self.horizontalLayout_2.addWidget(self.in_Ntcoil)

        self.si_Ntcoil = QSpinBox(self.widget)
        self.si_Ntcoil.setObjectName(u"si_Ntcoil")
        sizePolicy.setHeightForWidth(self.si_Ntcoil.sizePolicy().hasHeightForWidth())
        self.si_Ntcoil.setSizePolicy(sizePolicy)
        self.si_Ntcoil.setMinimumSize(QSize(100, 0))
        self.si_Ntcoil.setMaximumSize(QSize(100, 16777215))
        self.si_Ntcoil.setValue(99)

        self.horizontalLayout_2.addWidget(self.si_Ntcoil)

        self.verticalLayout_2.addLayout(self.horizontalLayout_2)

        self.in_qs = QLabel(self.widget)
        self.in_qs.setObjectName(u"in_qs")

        self.verticalLayout_2.addWidget(self.in_qs)

        self.in_Zs = QLabel(self.widget)
        self.in_Zs.setObjectName(u"in_Zs")

        self.verticalLayout_2.addWidget(self.in_Zs)

        self.in_Nlay = QLabel(self.widget)
        self.in_Nlay.setObjectName(u"in_Nlay")

        self.verticalLayout_2.addWidget(self.in_Nlay)

        self.groupBox = QGroupBox(self.widget)
        self.groupBox.setObjectName(u"groupBox")
        self.verticalLayout = QVBoxLayout(self.groupBox)
        self.verticalLayout.setObjectName(u"verticalLayout")
        self.out_Ntspc = QLabel(self.groupBox)
        self.out_Ntspc.setObjectName(u"out_Ntspc")

        self.verticalLayout.addWidget(self.out_Ntspc)

        self.out_Ncspc = QLabel(self.groupBox)
        self.out_Ncspc.setObjectName(u"out_Ncspc")

        self.verticalLayout.addWidget(self.out_Ncspc)

        self.verticalLayout_2.addWidget(self.groupBox)

        self.verticalSpacer = QSpacerItem(
            20, 247, QSizePolicy.Minimum, QSizePolicy.Expanding
        )

        self.verticalLayout_2.addItem(self.verticalSpacer)

        self.horizontalLayout_3.addWidget(self.widget)

        self.verticalLayout_3.addLayout(self.horizontalLayout_3)

        self.horizontalLayout_4 = QHBoxLayout()
        self.horizontalLayout_4.setObjectName(u"horizontalLayout_4")
        self.horizontalSpacer = QSpacerItem(
            40, 20, QSizePolicy.Expanding, QSizePolicy.Minimum
        )

        self.horizontalLayout_4.addItem(self.horizontalSpacer)

        self.b_previous = QPushButton(SWindParam)
        self.b_previous.setObjectName(u"b_previous")

        self.horizontalLayout_4.addWidget(self.b_previous)

        self.b_next = QPushButton(SWindParam)
        self.b_next.setObjectName(u"b_next")

        self.horizontalLayout_4.addWidget(self.b_next)

        self.verticalLayout_3.addLayout(self.horizontalLayout_4)

        self.retranslateUi(SWindParam)

        QMetaObject.connectSlotsByName(SWindParam)

    # setupUi

    def retranslateUi(self, SWindParam):
        SWindParam.setWindowTitle(
            QCoreApplication.translate("SWindParam", u"Form", None)
        )
        self.img_wind_geo.setText("")
        self.in_Npcpp.setText(QCoreApplication.translate("SWindParam", u"Npcpp", None))
        self.in_Ntcoil.setText(
            QCoreApplication.translate("SWindParam", u"Ntcoil", None)
        )
        # if QT_CONFIG(whatsthis)
        self.in_qs.setWhatsThis(
            QCoreApplication.translate("SWindParam", u"Number of phase", None)
        )
        # endif // QT_CONFIG(whatsthis)
        self.in_qs.setText(QCoreApplication.translate("SWindParam", u" qs : ?", None))
        # if QT_CONFIG(whatsthis)
        self.in_Zs.setWhatsThis(
            QCoreApplication.translate("SWindParam", u"Number of slot", None)
        )
        # endif // QT_CONFIG(whatsthis)
        self.in_Zs.setText(QCoreApplication.translate("SWindParam", u" Zs : ?", None))
        # if QT_CONFIG(whatsthis)
        self.in_Nlay.setWhatsThis(
            QCoreApplication.translate(
                "SWindParam", u"Number of winding layer in each slot", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        self.in_Nlay.setText(
            QCoreApplication.translate("SWindParam", u" Nlay : ?", None)
        )
        self.groupBox.setTitle(
            QCoreApplication.translate("SWindParam", u"Output", None)
        )
        # if QT_CONFIG(tooltip)
        self.out_Ntspc.setToolTip(
            QCoreApplication.translate(
                "SWindParam", u"Winding number of turns in series per phase", None
            )
        )
        # endif // QT_CONFIG(tooltip)
        # if QT_CONFIG(whatsthis)
        self.out_Ntspc.setWhatsThis(
            QCoreApplication.translate(
                "SWindParam", u"Winding number of turns in series per phase", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        self.out_Ntspc.setText(
            QCoreApplication.translate("SWindParam", u"Ntsp1 : ?", None)
        )
        # if QT_CONFIG(whatsthis)
        self.out_Ncspc.setWhatsThis(
            QCoreApplication.translate(
                "SWindParam", u"Number of coils in series per parallel circuit", None
            )
        )
        # endif // QT_CONFIG(whatsthis)
        self.out_Ncspc.setText(
            QCoreApplication.translate("SWindParam", u"Ncspc1 : ?", None)
        )
        self.b_previous.setText(
            QCoreApplication.translate("SWindParam", u"Previous", None)
        )
        self.b_next.setText(QCoreApplication.translate("SWindParam", u"Next", None))

    # retranslateUi



from ....Functions.Winding.gen_phase_list import gen_name
from SciDataTool import Data1D, DataLinspace, DataTime
from ....Functions.check_parent import check_parent


def get_wave(self):
    """Return the wave generated by the Drive

    Parameters
    ----------
    self : DriveWave
        A DriveWave object

    Returns
    -------
    wave : DataTime
        Voltage / current waveform (Nt, qs)
    """

    wave = self.wave.get_data()

    Phase = Data1D(
        name="phase",
        unit="",
        values=gen_name(wave.shape[0]),
        is_components=True,
    )
    # Ouput.Simulation.Electrical.EEC.Drive
    if (
        check_parent(self, 4)
        and self.parent.parent.parent.parent.elec.time is not None
        and len(self.parent.parent.parent.parent.elec.time) > 1
    ):
        Time = Data1D(
            name="time", unit="s", values=self.parent.parent.parent.parent.elec.time
        )
    else:
        Nt = wave.shape[1]
        Time = DataLinspace(
            name="time",
            unit="s",
            symmetries={},
            initial=0,
            final=Nt,
            number=Nt,
            include_endpoint=False,
        )

    if self.is_current:
        return DataTime(
            name="Current", unit="A", symbol="I", axes=[Phase, Time], values=wave
        )
    else:
        return DataTime(
            name="Voltage", unit="V", symbol="U", axes=[Phase, Time], values=wave
        )



# -*- coding: utf-8 -*-

from ....Methods.Machine.LamSlotMag import LMC_SlotTooLong


def check(self):
    """Check that the Lamination object is correct

    Parameters
    ----------
    self : LamSlotMag
        A LamSlotMag object

    Returns
    -------
    None

    Raises
    _______
    LMC_SlotTooLong
        The Slot is too long for the lamination (HYoke <0)
    """
    super(type(self), self).check()

    self.slot.check()

    if self.comp_height_yoke() < 0:
        raise LMC_SlotTooLong("The Slot is too long for the lamination " "(HYoke <0)")



# -*- coding: utf-8 -*-

import numpy as np


def get_real_point(self, vertice, ref_pt, nb_ref_pt=1):
    """Return the coordinates in the cell of a point in the reference cell.

    Parameters
     ----------
    self : RefTriangle3
         an RefCell object
    vertice : ndarray
        vertices of the cell
    ref_pt : ndarray
        ref point(s)
    nb_ref_pt : int
        nb of ref points

     Returns
     -------
    real_points : ndarray
        points coordinate

    """
    # TODO
    return None



# -*-- coding: utf-8 -*
def comp_length(self):
    """Returns the length of the PolarArc

    Parameters
    ----------
    self : PolarArc
        a PolarArc object


    Returns
    -------
    length : float
        length of the PolarArc [m]
    """
    # check if the PolarArc is correct
    self.check()

    lines = self.get_lines()
    length = 0
    for line in lines:
        length += line.comp_length()
    return length



# -*- coding: utf-8 -*-
from re import findall


def create_FEMM_magnet(femm, label, is_mmf, is_eddies, materials, lam):
    """Set the material of the magnet in FEMM

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    label : str
        label of the magnet
    is_mmfr : bool
        1 to compute the lamination magnetomotive force / magnetic field
    is_eddies : bool
        1 to calculate eddy currents
    materials : list
        list of materials already created in FEMM
    lam : LamSlotMag
        Lamination to set the magnet material

    Returns
    -------
    (str, list)
        property, materials

    """
    # some if's and else's to find the correct material parameter from magnet label
    idx_str = findall(r"_T\d+_", label)[0][2:-1]
    if "HoleMagnet" in label:
        magnet_dict = lam.hole[0].get_magnet_dict()
        magnet = magnet_dict["magnet_" + idx_str]
    else:
        magnet = lam.magnet
        # pole_mag = "_" + label[12] + "_" + label[-4]

    rho = magnet.mat_type.elec.rho  # Resistivity at 20°C
    Hcm = magnet.mat_type.mag.Hc  # Magnet coercitivity field
    mur = magnet.mat_type.mag.mur_lin

    if label not in materials:
        femm.mi_addmaterial(
            label,
            mur,
            mur,
            is_mmf * Hcm,
            0,
            is_mmf * is_eddies * 1e-6 / rho,
            0,
            0,
            1,
            0,
            0,
            0,
            0,
            0,
        )
        materials.append(label)

    return label, materials



# -*- coding: utf-8 -*-
from numpy import abs, newaxis, array


def get_cell_area(self, indices=None):
    """
    Return the area of the cells on the outer surface.
    #TODO address multiple cell type issue, i.e. distracted indices
    Parameters
    ----------
    self : MeshMat
        a MeshMat object
    indices : list
        list of the points to extract (optional)
    Returns
    -------
    areas: ndarray
        Area of the cells
    """
    logger = self.get_logger()
    area = []

    vertices_dict = self.get_vertice(indices=indices)

    for key, vertices in vertices_dict.items():
        if len(vertices) != 0:
            try:
                A = self.cell[key].interpolation.ref_cell.get_cell_area(vertices)
                A = A.tolist()
            except:
                logger.warning(
                    f'MeshMat: Reference Cell for "{key}" not found. '
                    + "Respective area set to zero."
                )
                A = [0 for i in range(vertices.shape[0])]

            area.extend(A)

    return array(area)



# -*- coding: utf-8 -*-
"""File generated according to SBar/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SBar.Ui_SBar import Ui_SBar


class Gen_SBar(Ui_SBar):
    def setupUi(self, SBar):
        """Abstract class to update the widget according to the csv doc"""
        Ui_SBar.setupUi(self, SBar)
        # Setup of in_Hscr
        txt = self.tr(u"""short circuit ring section radial height [m]""")
        self.in_Hscr.setWhatsThis(txt)
        self.in_Hscr.setToolTip(txt)

        # Setup of lf_Hscr
        self.lf_Hscr.validator().setBottom(0)
        txt = self.tr(u"""short circuit ring section radial height [m]""")
        self.lf_Hscr.setWhatsThis(txt)
        self.lf_Hscr.setToolTip(txt)

        # Setup of unit_Hscr
        txt = self.tr(u"""short circuit ring section radial height [m]""")
        self.unit_Hscr.setWhatsThis(txt)
        self.unit_Hscr.setToolTip(txt)

        # Setup of in_Lscr
        txt = self.tr(u"""short circuit ring section axial length""")
        self.in_Lscr.setWhatsThis(txt)
        self.in_Lscr.setToolTip(txt)

        # Setup of lf_Lscr
        self.lf_Lscr.validator().setBottom(0)
        txt = self.tr(u"""short circuit ring section axial length""")
        self.lf_Lscr.setWhatsThis(txt)
        self.lf_Lscr.setToolTip(txt)

        # Setup of unit_Lscr
        txt = self.tr(u"""short circuit ring section axial length""")
        self.unit_Lscr.setWhatsThis(txt)
        self.unit_Lscr.setToolTip(txt)

        # Setup of in_Lewout
        txt = self.tr(
            u"""straight length of the conductors outside the lamination before the curved part of winding overhang [m] - can be negative to tune the average turn length """
        )
        self.in_Lewout.setWhatsThis(txt)
        self.in_Lewout.setToolTip(txt)

        # Setup of lf_Lewout
        self.lf_Lewout.validator().setBottom(0)
        self.lf_Lewout.validator().setTop(100)
        txt = self.tr(
            u"""straight length of the conductors outside the lamination before the curved part of winding overhang [m] - can be negative to tune the average turn length """
        )
        self.lf_Lewout.setWhatsThis(txt)
        self.lf_Lewout.setToolTip(txt)

        # Setup of w_mat
        txt = self.tr(u"""Material of the Rotor short circuit ring""")
        self.w_mat.setWhatsThis(txt)
        self.w_mat.setToolTip(txt)



# -*- coding: utf-8 -*-


def comp_masses(self):
    """Compute the masses of the machine
    - Mmach : Mass total [kg]
    - Mfra : Mass of the Frame [kg]
    - Msha : Mass of the Shaft [kg]
    - Mrot : Mass dictionnary of the rotor masses
    - Msta : Mass dictionnary of the stator masses

    Parameters
    ----------
    self : Machine
        A Machine object

    Returns
    -------
    M_dict: dict
        A dictionnary of the Machine's masses (Mmach, Msha,
        Mfra, Mrot, Msta) [kg]

    """

    if self.frame is None:
        Mfra = 0
    else:
        Mfra = self.frame.comp_mass()
    if self.shaft is None:
        Msha = 0
    else:
        Msha = self.shaft.comp_mass()
    Mrot = self.rotor.comp_masses()
    Msta = self.stator.comp_masses()

    Mtot = Mfra + Msha + Mrot["Mtot"] + Msta["Mtot"]

    return {"Mmach": Mtot, "Mfra": Mfra, "Msha": Msha, "Mrot": Mrot, "Msta": Msta}



from numpy import diff, zeros, newaxis


def comp_emf(self):
    """Compute the Electromotive force [V]

    Parameters
    ----------
    self : OutMag
        an OutMag object

    """

    # Get stator winding flux
    Phi_wind = self.Phi_wind_stator
    phi_wind = Phi_wind.get_along("time[smallestperiod]", "phase")[Phi_wind.symbol]

    # Get time axis
    for axe in Phi_wind.axes:
        if axe.name == "time":
            Time = axe

    # Get time values on the smallest period
    _, is_antiper_t = Time.get_periodicity()
    time = Time.get_values(
        is_oneperiod=True,
        is_antiperiod=is_antiper_t,
    )

    # Calculate EMF and store it in OutMag
    if time.size > 1:
        emf = zeros(phi_wind.shape)
        emf[:-1, :] = diff(phi_wind, 1, 0) / diff(time, 1, 0)[:, newaxis]
        # We assume phi_wind to be periodic to compute the last value
        # and we assume time to be a linspace
        if is_antiper_t:
            sign0 = (
                -1
            )  # The opposite of the first value of the anti-period is after the last value of the anti-period
        else:
            sign0 = (
                1  # The first value of the period is after the last value of the period
            )
        emf[-1, :] = (sign0 * phi_wind[0, :] - phi_wind[-1, :]) / (time[1] - time[0])

        EMF = Phi_wind.copy()
        EMF.values = emf
        EMF.name = "Stator Winding Electromotive Force"
        EMF.unit = "V"
        EMF.symbol = "EMF"

        self.emf = EMF



# -*- coding: utf-8 -*-
from io import StringIO
from ....Methods.Elmer.Section import File, Variable, MATC

# some constants definition
WHITESPACE = " "
INDENT = 2 * WHITESPACE
COMMENT = "! "
NEWLINE = "\n"
END = "End"


def write(self, stream=None):
    """Write the Elmer SIF section to a stream. If no stream is given, the method
    will return a StringIO.
    """
    if not stream:
        stream = StringIO()

    # if section has an 'id' write a numbered sections
    if self.id:
        nbr = str(self.id)
    else:
        nbr = ""

    # write section header
    stream.write(NEWLINE)
    if self.comment:
        stream.write("! " + self.comment)
        stream.write(NEWLINE)
    stream.write(self.section + " " + nbr)

    # write section entries
    for key in self.keys():
        n = ""
        value, comment = self[key]
        if isinstance(value, list) and len(value) > 1:
            n = "(" + str(len(value)) + ")"
        stream.write(NEWLINE + INDENT)
        stream.write(key + n + " = " + _convert(value))
        if comment:
            stream.write(INDENT + "! " + comment)

    # end section
    stream.write(NEWLINE + END + NEWLINE)

    return stream


def _convert(value, get_type=True):
    """internal method to convert the data corresponding to Elmer SIF needs"""
    if isinstance(value, File):  # check before str due to inheretance
        return "File " * get_type + f'"{value}"'

    if isinstance(value, str):
        return "String " * get_type + f'"{value}"'

    if isinstance(value, float):
        return "Real " * get_type + f"{value}"

    if isinstance(value, bool):
        return "Logical " * get_type + f"{value}"

    if isinstance(value, int):
        return "Integer " * get_type + f"{value}"

    if isinstance(value, list):
        val_str = ""
        for idx, val in enumerate(value):
            get_type = True if idx == 0 else False
            val_str += _convert(val, get_type=get_type) + " "
        return val_str

    if isinstance(value, Variable):
        return "Variable " + value.name + NEWLINE + 2 * INDENT + _convert(value.value)

    if isinstance(value, MATC):
        return f'Real MATC "{value.expr}"'



# -*- coding: utf-8 -*-

import os
from os.path import join, isdir


def get_path_save_fea(self, output):
    """Return the path to save the Elmer simulation

    Parameters
    ----------
    self : MagElmer
        a MagElmer object
    output : Output
        an Output object

        Returns
    -------
    save_path: str
        Path to save the .fem file

    """
    save_dir = self.get_path_save(output)

    # if self.file_name not in [None, ""]:
    #     if self.file_name[-4:] != ".fem":
    #         file_name = self.file_name + ".fem"
    #     else:
    #         file_name = self.file_name
    # elif output.simu.machine.name not in [None, ""]:
    #     file_name = output.simu.machine.name + "_model.fem"
    # elif output.simu.name not in [None, ""]:
    #     file_name = output.simu.name + "_model.fem"
    # else:  # Default name
    #     file_name = "FEMM_simulation.fem"
    file_name = "ELMER_simulation"

    return join(save_dir, file_name)



# -*- coding: utf-8 -*-

from ....GUI.Dialog.DMachineSetup.SMachineType.SMachineType import SMachineType
from ....GUI.Dialog.DMachineSetup.SMachineDimension.SMachineDimension import (
    SMachineDimension,
)
from ....GUI.Dialog.DMachineSetup.SLamParam.SLamParam import SLamParam
from ....GUI.Dialog.DMachineSetup.SWindPat.SWindPat import SWindPat
from ....GUI.Dialog.DMachineSetup.SWindParam.SWindParam import SWindParam
from ....GUI.Dialog.DMachineSetup.SWindCond.SWindCond import SWindCond
from ....GUI.Dialog.DMachineSetup.SWSlot.SWSlot import SWSlot
from ....GUI.Dialog.DMachineSetup.SWPole.SWPole import SWPole
from ....GUI.Dialog.DMachineSetup.SBar.SBar import SBar
from ....GUI.Dialog.DMachineSetup.SMHoleMag.SMHoleMag import SMHoleMag
from ....GUI.Dialog.DMachineSetup.SMSlot.SMSlot import SMSlot

from ....Classes.Magnet import Magnet
from ....Classes.LamHole import LamHole
from ....Classes.LamSlot import LamSlot
from ....Classes.LamSlotMag import LamSlotMag
from ....Classes.LamSlotWind import LamSlotWind
from ....Classes.LamSquirrelCage import LamSquirrelCage
from ....Classes.MachineIPMSM import MachineIPMSM
from ....Classes.MachineSyRM import MachineSyRM
from ....Classes.MachineDFIM import MachineDFIM
from ....Classes.MachineSCIM import MachineSCIM
from ....Classes.MachineSIPMSM import MachineSIPMSM
from ....Classes.MachineWRSM import MachineWRSM
from ....Classes.MachineSRM import MachineSRM
from ....Classes.SlotM10 import SlotM10
from ....Classes.HoleM50 import HoleM50
from ....Classes.WindingDW2L import WindingDW2L
from ....Classes.WindingCW2LT import WindingCW2LT
from ....Classes.WindingSC import WindingSC
from ....GUI.Resources import pixmap_dict

# Steps needed to setup a LamSlotWind
LSW_step = [SLamParam, SWSlot, SWindPat, SWindParam, SWindCond]
# Steps needed to setup a LamSlotWind for the rotor of a WRSM
LP_step = [SLamParam, SWPole, SWindPat, SWindParam, SWindCond]
# Steps needed to setup a LamSquirrelCage
LSC_step = [SLamParam, SWSlot, SBar]
# Steps needed to setup a LamHole
LH_step = [SLamParam, SMHoleMag]
# Steps needed to setup a LamSlot
LS_step = [SLamParam, SWSlot]
# Steps needed to setup a LamSlotMag
LSM_step = [SLamParam, SMSlot]
# Steps to start the design of a machine with 2 laminations
S_step = [SMachineType, SMachineDimension]

# Defaut machine for initialization
machine1 = MachineSCIM()
machine1.stator = LamSlotWind()
machine1.stator.winding = WindingDW2L()
machine1.rotor = LamSquirrelCage()
machine1.rotor.winding = WindingSC()
machine1._set_None()  # Empty machine
machine1.type_machine = 1
machine1.stator.is_stator = True
machine1.rotor.is_stator = False

machine4 = MachineDFIM()
machine4.stator = LamSlotWind()
machine4.stator.winding = WindingDW2L()
machine4.rotor = LamSlotWind()
machine4.rotor.winding = WindingDW2L()
machine4._set_None()  # Empty machine
machine4.type_machine = 4
machine4.stator.is_stator = True
machine4.rotor.is_stator = False

machine5 = MachineSyRM()
machine5.stator = LamSlotWind()
machine5.stator.winding = WindingDW2L()
machine5.rotor = LamHole()
machine5.rotor.hole = list()
machine5.rotor.hole.append(HoleM50())
machine5.rotor.hole[0].remove_magnet()
machine5._set_None()  # Empty machine
machine5.type_machine = 5
machine5.stator.is_stator = True
machine5.rotor.is_stator = False

machine7 = MachineSIPMSM()
machine7.stator = LamSlotWind()
machine7.stator.winding = WindingDW2L()
machine7.rotor = LamSlotMag()
machine7.rotor.slot = SlotM10()
machine7._set_None()  # Empty machine
machine7.type_machine = 7
machine7.stator.is_stator = True
machine7.rotor.is_stator = False

machine8 = MachineIPMSM()
machine8.stator = LamSlotWind()
machine8.stator.winding = WindingDW2L()
machine8.rotor = LamHole()
machine8.rotor.hole = list()
machine8.rotor.hole.append(HoleM50())
machine8._set_None()  # Empty machine
machine8.type_machine = 8
machine8.stator.is_stator = True
machine8.rotor.is_stator = False

machine9 = MachineWRSM()
machine9.stator = LamSlotWind()
machine9.stator.winding = WindingDW2L()
machine9.rotor = LamSlotWind()
machine9.rotor.winding = WindingCW2LT()
machine9._set_None()  # Empty machine
machine9.type_machine = 9
machine9.stator.is_stator = True
machine9.rotor.is_stator = False

machine10 = MachineSRM()
machine10.stator = LamSlotWind()
machine10.stator.winding = WindingDW2L()
machine10.rotor = LamSlot()
machine10._set_None()  # Empty machine
machine10.type_machine = 10
machine10.stator.is_stator = True
machine10.rotor.is_stator = False

# Dictionnary with all the information to set a SCIM
SCIM_dict = {
    "machine_type": MachineSCIM,
    "init_machine": machine1,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LSC_step,
    "name": "SCIM",
    "img": pixmap_dict["SCIM"],
    "txt": "SCIM (Squirrel Cage Induction Machine)",
}
# Dictionnary with all the information to set a DFIM
DFIM_dict = {
    "machine_type": MachineDFIM,
    "init_machine": machine4,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LSW_step,
    "name": "DFIM",
    "img": pixmap_dict["DFIM"],
    "txt": "DFIM (Doubly Fed Induction Machine)",
}
# Dictionnary with all the information to set a SynRM
SynRM_dict = {
    "machine_type": MachineSyRM,
    "init_machine": machine5,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LH_step,
    "name": "SynRM",
    "img": pixmap_dict["SynRM"],
    "txt": "SynRM (Synchronous Reluctance Machine)",
}
# Dictionnary with all the information to set a SIPMSM
SIPMSM_dict = {
    "machine_type": MachineSIPMSM,
    "init_machine": machine7,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LSM_step,
    "name": "SPMSM",
    "img": pixmap_dict["SPMSM"],
    "txt": "SPMSM (Surface Permanent Magnet Synchronous Machine)",
}
# Dictionnary with all the information to set a IPMSM
IPMSM_dict = {
    "machine_type": MachineIPMSM,
    "init_machine": machine8,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LH_step,
    "name": "IPMSM",
    "img": pixmap_dict["IPMSM"],
    "txt": "IPMSM (Interior Permanent Magnet Synchronous Machine)",
}
# Dictionnary with all the information to set a WRSM
WRSM_dict = {
    "machine_type": MachineWRSM,
    "init_machine": machine9,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LP_step,
    "name": "WRSM",
    "img": pixmap_dict["WRSM"],
    "txt": "WRSM (Wound Rotor Synchronous Machine)",
}
# Dictionnary with all the information to set a SCIM
SRM_dict = {
    "machine_type": MachineSRM,
    "init_machine": machine10,
    "start_step": S_step,
    "stator_step": LSW_step,
    "rotor_step": LS_step,
    "name": "SRM",
    "img": pixmap_dict["SCIM"],
    "txt": "SRM (Switched Reluctance Machine)",
}
# List of machine types available in the GUI
mach_list = [
    SCIM_dict,
    DFIM_dict,
    SynRM_dict,
    SIPMSM_dict,
    IPMSM_dict,
    WRSM_dict,
    SRM_dict,
]
# To find the correct index according to the machine
mach_index = [mach_dict["machine_type"] for mach_dict in mach_list]



# Default config_dict
default_config_dict = {"MAIN": {}, "GUI": {}, "PLOT": {}}

default_config_dict["MAIN"]["MACHINE_DIR"] = ""
default_config_dict["MAIN"]["MATLIB_DIR"] = ""

default_config_dict["GUI"]["UNIT_M"] = 1  # length unit: 0 for m, 1 for mm
default_config_dict["GUI"]["UNIT_M2"] = 1  # Surface unit: 0 for m^2, 1 for mm^2
default_config_dict["GUI"]["CSS_NAME"] = "pyleecan.css"
default_config_dict["GUI"]["CSS_PATH"] = ""
# Name of the color set to use
default_config_dict["PLOT"]["COLOR_DICT_NAME"] = "pyleecan_color.json"
default_config_dict["PLOT"]["COLOR_DICT"] = {}
default_config_dict["PLOT"]["FONT_NAME"] = "Arial"
default_config_dict["PLOT"]["FONT_SIZE_TITLE"] = 12
default_config_dict["PLOT"]["FONT_SIZE_LABEL"] = 10
default_config_dict["PLOT"]["FONT_SIZE_LEGEND"] = 8
default_config_dict["PLOT"]["LINE_STYLE"] = ["solid", "dashed", "dashdot", "dotted"]



# -*- coding: utf-8 -*-


def comp_surface_active(self):
    """Compute the Slot active inner surface (by analytical computation)

    Parameters
    ----------
    self : SlotM10
        A SlotM10 object

    Returns
    -------
    Swind: float
        Slot active inner surface [m**2]

    """

    return self.Hmag * self.Wmag



# -*- coding: utf-8 -*-

from numpy import (
    pi,
    column_stack,
    exp,
    real,
    imag,
    cos,
    sin,
    abs as np_abs,
    angle as np_angle,
)


class DimError(Exception):
    """Raised when the dimension of the field does not match requested operation (3D vs 1D)"""

    pass


def xyz_to_rphiz(values):
    """Converts axis values from cartesian coordinates into cylindrical coordinates

    Parameters
    ----------
    values: array
        Values of the axis to convert (Nx3)
    Returns
    -------
    ndarray of the axis (Nx3)
    """

    x = values[:, 0]
    y = values[:, 1]
    z = values[:, 2]

    affixe = x + 1j * y
    r = np_abs(affixe)
    phi = (np_angle(affixe) + 2 * pi) % (2 * pi)

    return column_stack((r, phi, z))


def rphiz_to_xyz(values):
    """Converts axis values from cylindrical coordinates into cartesian coordinates

    Parameters
    ----------
    values: array
        Values of the axis to convert (Nx3)
    Returns
    -------
    ndarray of the axis (Nx3)
    """

    r = values[:, 0]
    phi = values[:, 1]
    z = values[:, 2]

    affixe = r * exp(1j * phi)
    x = real(affixe)
    y = imag(affixe)

    return column_stack((x, y, z))


def xyz_to_rphiz_field(values, phi):
    """Converts field values from cartesian coordinates into cylindrical coordinates

    Parameters
    ----------
    values: array
        Values of the field to convert (Nx3)
    phi: array
        Values of the angle axis (N)
    Returns
    -------
    ndarray of the field (Nx3)
    """

    field_x = values[:, 0]
    field_y = values[:, 1]
    field_z = values[:, 2]

    cos_phi = cos(phi)
    sin_phi = sin(phi)

    field_r = cos_phi * field_x + sin_phi * field_y
    field_phi = -sin_phi * field_x + cos_phi * field_y

    return column_stack((field_r, field_phi, field_z))


def rphiz_to_xyz_field(values, phi):
    """Converts field values from cylindrical coordinates into cartesian coordinates

    Parameters
    ----------
    values: array
        Values of the field to convert (Nx3)
    phi: array
        Values of the angle axis (N)
    Returns
    -------
    ndarray of the field (Nx3)
    """

    field_r = values[:, 0]
    field_phi = values[:, 1]
    field_z = values[:, 2]

    cos_phi = cos(phi)
    sin_phi = sin(phi)

    field_x = cos_phi * field_r - sin_phi * field_phi
    field_y = sin_phi * field_r + cos_phi * field_phi

    return column_stack((field_x, field_y, field_z))


def cart2pol(values, points):
    """Converts field values from cartesian coordinates into cylindrical coordinates

    Parameters
    ----------
    values: array
        Values of the field to convert (Nx3)
    phi: array
        Values of the angle axis (N)
    Returns
    -------
    ndarray of the field (Nx3)
    """

    phi = xyz_to_rphiz(points)[:, 1]
    return xyz_to_rphiz_field(values, phi)


def pol2cart(values, points):
    """Converts field values from cylindrical coordinates into cartesian coordinates

    Parameters
    ----------
    values: array
        Values of the field to convert (Nx3)
    phi: array
        Values of the angle axis (N)
    Returns
    -------
    ndarray of the field (Nx3)
    """

    phi = xyz_to_rphiz(points)[:, 1]
    return rphiz_to_xyz_field(values, phi)



from ....Functions.Load.import_class import import_class


class VarSimuError(Exception):
    pass


def check_param(self):
    """Check VarParam parameters validity"""
    if type(self).__name__ == "VarSimu":
        raise VarSimuError(
            "VarSimu is an abstract class, please create one of its daughters."
        )

    Simulation = import_class("pyleecan.Classes", "Simulation")
    # Check for simulation or VarSimu
    if self.parent is None or not isinstance(self.parent, Simulation):
        raise VarSimuError("VarSimu object must be inside a Simulation object")



class PointArc3Error(Exception):
    """ """

    pass


class NbPointArc3DError(Exception):
    """ """

    pass


class AngleRotationArc3Error(Exception):
    """ """

    pass


class PointTranslateArc3Error(Exception):
    """ """

    pass



from numpy import sqrt, array
from numpy import arccos, array, dot, pi, cross
from numpy.linalg import det, norm


def distance_numpy(A, B, P):
    # from: https://gist.github.com/nim65s/5e9902cd67f094ce65b0
    """ segment line AB, point P, where each one is an array([x, y]) """
    if all(A == P) or all(B == P):
        return 0
    if arccos(dot((P - A) / norm(P - A), (B - A) / norm(B - A))) > pi / 2:
        return norm(P - A)
    if arccos(dot((P - B) / norm(P - B), (A - B) / norm(A - B))) > pi / 2:
        return norm(P - B)
    return norm(cross(A - B, A - P)) / norm(B - A)


def comp_distance(self, Z):
    """Compute the distance of a point to the Segment

    Parameters
    ----------
    self : Segment
        A Segment object
    Z : complex
        Complex coordinate of the point

    Returns
    -------
    D : float
        distance of a point to the Segment
    """

    Z1 = array([self.begin.real, self.begin.imag])
    Z2 = array([self.end.real, self.end.imag])
    Z3 = array([Z.real, Z.imag])
    return distance_numpy(Z1, Z2, Z3)



# -*- coding: utf-8 -*-

from ..init_fig import init_fig
from .plot_2D import plot_2D


def plot_A_quiver_2D(
    data,
    t=None,
    t_index=0,
    is_norm=False,
    unit="SI",
    is_show_fig=None,
    save_path=None,
    fig=None,
    ax=None,
):
    """Plots a 2D vector field

    Parameters
    ----------
    data : Data
        a Data object
    t : float
        time value at which to slice
    t_index : int
        time index at which to slice
    is_norm : bool
        boolean indicating if the field must be normalized
    unit : str
        unit in which to plot the field
    is_show_fig : bool
        True to show figure after plot
    save_path : str
        full path including folder, name and extension of the file to save if save_path is not None
    fig : Matplotlib.figure.Figure
        existing figure to use if None create a new one
    ax : Matplotlib.axes.Axes object
        ax on which to plot the data
    """

    # Set figure if needed
    if fig is None and ax is None:
        (fig, ax, _, _) = init_fig(fig=None, shape="rectangle")

    xlabel = "x [m]"
    ylabel = "y [m]"
    # Prepare the extractions
    if t != None:
        t_str = "time=" + str(t)
    else:
        t_str = "time[" + str(t_index) + "]"
    title = data.name + " over space at " + t_str

    # Extract the fields
    results = data.get_along("xy", t_str, unit=unit, is_norm=is_norm)
    xy = results["xy"]
    Ydata = results[data.symbol]

    # Plot the original graph
    plot_2D(
        xy,
        [Ydata],
        fig=fig,
        ax=ax,
        title=title,
        xlabel=xlabel,
        ylabel=ylabel,
        type="quiver",
        is_show_fig=is_show_fig,
        save_path=save_path,
    )



# -*- coding: utf-8 -*-


def get_m(self, value):
    """Convert the value the correct m unit (to display in GUI)

    Parameters
    ----------
    self : Unit
        A Unit object
    value : float
        Value to convert

    Returns
    -------
    value : float
        Converted value in the current unit

    """
    if value is None:
        return None
    if self.unit_m == 1:  # Convert to mm
        return value * 1000
    else:
        return value



# -*- coding: utf-8 -*-


def create_FEMM_boundary_conditions(femm, sym, is_antiper):
    """Create the boundary conditions in FEMM

    Parameters
    ----------
    femm : FEMMHandler
        client to send command to a FEMM instance
    sym : int
        Symmetry factor of the machine
    is_antiper : bool
        True if an anti-periodicity is considered

    Returns
    -------
    None
    """
    if sym == 1:
        is_antiper = False

    # anti periodic boundary conditions
    if is_antiper:
        BdPr = 5
    # even number of rotor poles /slots -> periodic boundary conditions
    else:
        BdPr = 4

    # Dirichlet (no flux going out)
    femm.mi_addboundprop("bc_A0", 0, 0, 0, 0, 0, 0, 0, 0, 0)
    # periodic and anti periodic conditions
    femm.mi_addboundprop("bc_ag2", 0, 0, 0, 0, 0, 0, 0, 0, BdPr + 2)
    if sym > 1:
        femm.mi_addboundprop("bc_s1", 0, 0, 0, 0, 0, 0, 0, 0, BdPr)
        femm.mi_addboundprop("bc_ag1", 0, 0, 0, 0, 0, 0, 0, 0, BdPr)
        femm.mi_addboundprop("bc_ag3", 0, 0, 0, 0, 0, 0, 0, 0, BdPr)
        femm.mi_addboundprop("bc_r1", 0, 0, 0, 0, 0, 0, 0, 0, BdPr)
        femm.mi_addboundprop("bc_r2", 0, 0, 0, 0, 0, 0, 0, 0, BdPr)



# -*- coding: utf-8 -*-

import pyvista as pv
import meshio
from os import makedirs, remove
from os.path import isdir, split
from pyleecan.definitions import RESULT_DIR


def get_mesh_pv(self, path=RESULT_DIR + "/temp.vtk", indices=None):
    """Return the pyvista mesh object (or submesh).

    Parameters
    ----------
    self : MeshMat
        a MeshMat object
    indices : list
        list of the points to extract (optional)

    Returns
    -------
    mesh : pyvista.core.pointset.UnstructuredGrid
        a pyvista UnstructuredGrid object
    """

    points = self.get_point()
    cells, _, _ = self.get_cell()

    cells_meshio = list()
    for key in cells:
        cells_meshio.append((key, cells[key]))
        # Write .vtk file using meshio

    # Make sure that the file exists
    if not isdir(split(path)[0]):
        makedirs(split(path)[0])

    meshio.write_points_cells(filename=path, points=points, cells=cells_meshio)

    # Read .vtk file with pyvista
    mesh = pv.read(path)

    # Extract submesh
    if indices is not None:
        mesh = mesh.extract_points(indices)

    remove(path)

    return mesh



# -*- coding: utf-8 -*-


def comp_height(self):
    """Compute the height of the Slot.
    Caution, the bottom of the Slot is an Arc

    Parameters
    ----------
    self : SlotW61
        A SlotW61 object

    Returns
    -------
    Htot: float
        Height of the slot [m]

    """

    Rbo = self.get_Rbo()
    [Z1, Z2, Z3, Z4, Z5, Z6, Z7, Z8, Z9, Z10] = self._comp_point_coordinate()

    return Rbo - abs(Z5)



# -*- coding: utf-8 -*-

"""File generated by generate_code() - 
WARNING! All changes made in this file will be lost!
"""

from ..Classes.Arc import Arc
from ..Classes.Arc1 import Arc1
from ..Classes.Arc2 import Arc2
from ..Classes.Arc3 import Arc3
from ..Classes.Bore import Bore
from ..Classes.BoreFlower import BoreFlower
from ..Classes.BoreUD import BoreUD
from ..Classes.CellMat import CellMat
from ..Classes.Circle import Circle
from ..Classes.CondType11 import CondType11
from ..Classes.CondType12 import CondType12
from ..Classes.CondType21 import CondType21
from ..Classes.CondType22 import CondType22
from ..Classes.Conductor import Conductor
from ..Classes.DXFImport import DXFImport
from ..Classes.DataKeeper import DataKeeper
from ..Classes.Drive import Drive
from ..Classes.DriveWave import DriveWave
from ..Classes.EEC import EEC
from ..Classes.EEC_PMSM import EEC_PMSM
from ..Classes.EEC_SCIM import EEC_SCIM
from ..Classes.Electrical import Electrical
from ..Classes.Elmer import Elmer
from ..Classes.ElmerResults import ElmerResults
from ..Classes.ElmerResultsVTU import ElmerResultsVTU
from ..Classes.FPGNSeg import FPGNSeg
from ..Classes.FPGNTri import FPGNTri
from ..Classes.FluxLink import FluxLink
from ..Classes.FluxLinkFEMM import FluxLinkFEMM
from ..Classes.Force import Force
from ..Classes.ForceMT import ForceMT
from ..Classes.Frame import Frame
from ..Classes.GUIOption import GUIOption
from ..Classes.GaussPoint import GaussPoint
from ..Classes.Hole import Hole
from ..Classes.HoleM50 import HoleM50
from ..Classes.HoleM51 import HoleM51
from ..Classes.HoleM52 import HoleM52
from ..Classes.HoleM53 import HoleM53
from ..Classes.HoleM54 import HoleM54
from ..Classes.HoleM57 import HoleM57
from ..Classes.HoleM58 import HoleM58
from ..Classes.HoleMag import HoleMag
from ..Classes.HoleUD import HoleUD
from ..Classes.Import import Import
from ..Classes.ImportData import ImportData
from ..Classes.ImportGenMatrixSin import ImportGenMatrixSin
from ..Classes.ImportGenToothSaw import ImportGenToothSaw
from ..Classes.ImportGenVectLin import ImportGenVectLin
from ..Classes.ImportGenVectSin import ImportGenVectSin
from ..Classes.ImportMatlab import ImportMatlab
from ..Classes.ImportMatrix import ImportMatrix
from ..Classes.ImportMatrixVal import ImportMatrixVal
from ..Classes.ImportMatrixXls import ImportMatrixXls
from ..Classes.ImportVectorField import ImportVectorField
from ..Classes.IndMag import IndMag
from ..Classes.IndMagFEMM import IndMagFEMM
from ..Classes.Input import Input
from ..Classes.InputCurrent import InputCurrent
from ..Classes.InputElec import InputElec
from ..Classes.InputFlux import InputFlux
from ..Classes.InputForce import InputForce
from ..Classes.Interpolation import Interpolation
from ..Classes.LamHole import LamHole
from ..Classes.LamSlot import LamSlot
from ..Classes.LamSlotMag import LamSlotMag
from ..Classes.LamSlotMulti import LamSlotMulti
from ..Classes.LamSlotWind import LamSlotWind
from ..Classes.LamSquirrelCage import LamSquirrelCage
from ..Classes.Lamination import Lamination
from ..Classes.Line import Line
from ..Classes.Loss import Loss
from ..Classes.LossModel import LossModel
from ..Classes.LossModelBertotti import LossModelBertotti
from ..Classes.LossModelWinding import LossModelWinding
from ..Classes.Machine import Machine
from ..Classes.MachineAsync import MachineAsync
from ..Classes.MachineDFIM import MachineDFIM
from ..Classes.MachineIPMSM import MachineIPMSM
from ..Classes.MachineSCIM import MachineSCIM
from ..Classes.MachineSIPMSM import MachineSIPMSM
from ..Classes.MachineSRM import MachineSRM
from ..Classes.MachineSyRM import MachineSyRM
from ..Classes.MachineSync import MachineSync
from ..Classes.MachineUD import MachineUD
from ..Classes.MachineWRSM import MachineWRSM
from ..Classes.MagElmer import MagElmer
from ..Classes.MagFEMM import MagFEMM
from ..Classes.Magnet import Magnet
from ..Classes.Magnetics import Magnetics
from ..Classes.MatEconomical import MatEconomical
from ..Classes.MatElectrical import MatElectrical
from ..Classes.MatHT import MatHT
from ..Classes.MatMagnetics import MatMagnetics
from ..Classes.MatStructural import MatStructural
from ..Classes.Material import Material
from ..Classes.Mesh import Mesh
from ..Classes.MeshMat import MeshMat
from ..Classes.MeshSolution import MeshSolution
from ..Classes.MeshVTK import MeshVTK
from ..Classes.Mode import Mode
from ..Classes.Notch import Notch
from ..Classes.NotchEvenDist import NotchEvenDist
from ..Classes.OptiConstraint import OptiConstraint
from ..Classes.OptiDesignVar import OptiDesignVar
from ..Classes.OptiGenAlg import OptiGenAlg
from ..Classes.OptiGenAlgNsga2Deap import OptiGenAlgNsga2Deap
from ..Classes.OptiObjective import OptiObjective
from ..Classes.OptiProblem import OptiProblem
from ..Classes.OptiSolver import OptiSolver
from ..Classes.OutElec import OutElec
from ..Classes.OutForce import OutForce
from ..Classes.OutGeo import OutGeo
from ..Classes.OutGeoLam import OutGeoLam
from ..Classes.OutInternal import OutInternal
from ..Classes.OutLoss import OutLoss
from ..Classes.OutMag import OutMag
from ..Classes.OutMagElmer import OutMagElmer
from ..Classes.OutMagFEMM import OutMagFEMM
from ..Classes.OutPost import OutPost
from ..Classes.OutStruct import OutStruct
from ..Classes.Output import Output
from ..Classes.ParamExplorer import ParamExplorer
from ..Classes.ParamExplorerSet import ParamExplorerSet
from ..Classes.PointMat import PointMat
from ..Classes.PolarArc import PolarArc
from ..Classes.Post import Post
from ..Classes.PostFunction import PostFunction
from ..Classes.PostMethod import PostMethod
from ..Classes.PostPlot import PostPlot
from ..Classes.RefCell import RefCell
from ..Classes.RefLine3 import RefLine3
from ..Classes.RefQuad4 import RefQuad4
from ..Classes.RefQuad9 import RefQuad9
from ..Classes.RefSegmentP1 import RefSegmentP1
from ..Classes.RefTriangle3 import RefTriangle3
from ..Classes.RefTriangle6 import RefTriangle6
from ..Classes.ScalarProduct import ScalarProduct
from ..Classes.ScalarProductL2 import ScalarProductL2
from ..Classes.Section import Section
from ..Classes.Segment import Segment
from ..Classes.Shaft import Shaft
from ..Classes.Simu1 import Simu1
from ..Classes.Simulation import Simulation
from ..Classes.Slot import Slot
from ..Classes.Slot19 import Slot19
from ..Classes.SlotCirc import SlotCirc
from ..Classes.SlotDC import SlotDC
from ..Classes.SlotM10 import SlotM10
from ..Classes.SlotM11 import SlotM11
from ..Classes.SlotM12 import SlotM12
from ..Classes.SlotM13 import SlotM13
from ..Classes.SlotM14 import SlotM14
from ..Classes.SlotM15 import SlotM15
from ..Classes.SlotM16 import SlotM16
from ..Classes.SlotM17 import SlotM17
from ..Classes.SlotUD import SlotUD
from ..Classes.SlotUD2 import SlotUD2
from ..Classes.SlotW10 import SlotW10
from ..Classes.SlotW11 import SlotW11
from ..Classes.SlotW12 import SlotW12
from ..Classes.SlotW13 import SlotW13
from ..Classes.SlotW14 import SlotW14
from ..Classes.SlotW15 import SlotW15
from ..Classes.SlotW16 import SlotW16
from ..Classes.SlotW21 import SlotW21
from ..Classes.SlotW22 import SlotW22
from ..Classes.SlotW23 import SlotW23
from ..Classes.SlotW24 import SlotW24
from ..Classes.SlotW25 import SlotW25
from ..Classes.SlotW26 import SlotW26
from ..Classes.SlotW27 import SlotW27
from ..Classes.SlotW28 import SlotW28
from ..Classes.SlotW29 import SlotW29
from ..Classes.SlotW60 import SlotW60
from ..Classes.SlotW61 import SlotW61
from ..Classes.Solution import Solution
from ..Classes.SolutionData import SolutionData
from ..Classes.SolutionMat import SolutionMat
from ..Classes.SolutionVector import SolutionVector
from ..Classes.SolverInputFile import SolverInputFile
from ..Classes.StructElmer import StructElmer
from ..Classes.Structural import Structural
from ..Classes.SurfLine import SurfLine
from ..Classes.SurfRing import SurfRing
from ..Classes.Surface import Surface
from ..Classes.Trapeze import Trapeze
from ..Classes.Unit import Unit
from ..Classes.VarLoad import VarLoad
from ..Classes.VarLoadCurrent import VarLoadCurrent
from ..Classes.VarParam import VarParam
from ..Classes.VarSimu import VarSimu
from ..Classes.VentilationCirc import VentilationCirc
from ..Classes.VentilationPolar import VentilationPolar
from ..Classes.VentilationTrap import VentilationTrap
from ..Classes.Winding import Winding
from ..Classes.WindingCW1L import WindingCW1L
from ..Classes.WindingCW2LR import WindingCW2LR
from ..Classes.WindingCW2LT import WindingCW2LT
from ..Classes.WindingDW1L import WindingDW1L
from ..Classes.WindingDW2L import WindingDW2L
from ..Classes.WindingSC import WindingSC
from ..Classes.WindingUD import WindingUD
from ..Classes.XOutput import XOutput



# -*- coding: utf-8 -*-

from ....Classes.LamSlotWind import LamSlotWind
from ....Methods.Machine import MachineCheckError


def check(self):
    """Check that the Machine object is correct

    Parameters
    ----------
    self : MachineDFIM
        A MachineDFIM object

    Returns
    -------
    None

    Raises
    _______
    M4C_WrongRotor
        The Rotor of a MachineDFIM must be a LamSlotWind
    M4C_WrongStator
        The Stator of a MachineDFIM must be a LamSlotWind
    M4C_PError
        The stator and the rotor winding must have the same value for p
    """

    super(type(self), self).check()

    if not isinstance(self.rotor, LamSlotWind):
        raise M4C_WrongRotor("The Rotor of a MachineDFIM must be a LamSlotWind")
    if not isinstance(self.stator, LamSlotWind):
        raise M4C_WrongStator("The Stator of a MachineDFIM must be a " "LamSlotWind")
    if self.rotor.winding.p != self.stator.winding.p:
        raise M4C_PError(
            "The stator and the rotor winding must have the " "same value for p"
        )


class M4C_WrongRotor(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    with
        Winding

    """

    pass


class M4C_WrongStator(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    with
        Winding

    """

    pass


class M4C_PError(MachineCheckError):
    """

    Parameters
    ----------

    Returns
    -------

    Raises
    ------
    different
        p value

    """

    pass



# -*- coding: utf-8 -*-
from io import StringIO
from .... import __version__

# some constants definition
WHITESPACE = " "
INDENT = 2 * WHITESPACE
COMMENT = "! "
NEWLINE = "\n"
END = "End"

MESH_FILE = "Mesh"


def write(self, stream=None):
    """Write the Elmer Solver Input File. If no stream is given, the method
    will return a StringIO.
    """
    if not stream:
        stream = StringIO()

    # write SIF header
    _write_header(stream)

    for section in self.sections:
        section.write(stream)

    return stream


def _write_header(f):
    f.write(COMMENT + f"-" * 80 + NEWLINE)
    f.write(COMMENT + f"File Generated by pyleecan v{__version__}" + NEWLINE)
    f.write(COMMENT + f"-" * 80 + NEWLINE)

    f.write(NEWLINE)
    f.write(f"Header")
    f.write(NEWLINE + INDENT)
    f.write(f"CHECK KEYWORDS Warn")
    f.write(NEWLINE + INDENT)
    f.write(f'Mesh DB "." "{MESH_FILE}"')
    f.write(NEWLINE + INDENT)
    f.write(f'Include Path ""')
    f.write(NEWLINE + INDENT)
    f.write(f'Results Directory ""')
    f.write(NEWLINE + END + NEWLINE)



from numpy import sqrt


def comp_height_active(self):
    """Compute the height of the winding area

    Parameters
    ----------
    self : SlotDC
        A SlotDC object

    Returns
    -------
    Hwind: float
        Height of the winding area [m]

    """

    return self.R3 + self.H3 + self.H2 + sqrt((self.D1 / 2) ** 2 - (self.W1 / 2) ** 2)



# -*- coding: utf-8 -*-
"""File generated according to PWSlot60/gen_list.json
WARNING! All changes made in this file will be lost!
"""
from pyleecan.GUI.Dialog.DMachineSetup.SWPole.PWSlot60.Ui_PWSlot60 import Ui_PWSlot60


class Gen_PWSlot60(Ui_PWSlot60):
    def setupUi(self, PWSlot60):
        """Abstract class to update the widget according to the csv doc"""
        Ui_PWSlot60.setupUi(self, PWSlot60)
        # Setup of in_R1
        txt = self.tr(u"""Pole top radius""")
        self.in_R1.setWhatsThis(txt)
        self.in_R1.setToolTip(txt)

        # Setup of lf_R1
        self.lf_R1.validator().setBottom(0)
        txt = self.tr(u"""Pole top radius""")
        self.lf_R1.setWhatsThis(txt)
        self.lf_R1.setToolTip(txt)

        # Setup of in_W1
        txt = self.tr(u"""Pole top width""")
        self.in_W1.setWhatsThis(txt)
        self.in_W1.setToolTip(txt)

        # Setup of lf_W1
        self.lf_W1.validator().setBottom(0)
        txt = self.tr(u"""Pole top width""")
        self.lf_W1.setWhatsThis(txt)
        self.lf_W1.setToolTip(txt)

        # Setup of in_W2
        txt = self.tr(u"""Pole bottom width""")
        self.in_W2.setWhatsThis(txt)
        self.in_W2.setToolTip(txt)

        # Setup of lf_W2
        self.lf_W2.validator().setBottom(0)
        txt = self.tr(u"""Pole bottom width""")
        self.lf_W2.setWhatsThis(txt)
        self.lf_W2.setToolTip(txt)

        # Setup of in_H1
        txt = self.tr(u"""Pole top height""")
        self.in_H1.setWhatsThis(txt)
        self.in_H1.setToolTip(txt)

        # Setup of lf_H1
        self.lf_H1.validator().setBottom(0)
        txt = self.tr(u"""Pole top height""")
        self.lf_H1.setWhatsThis(txt)
        self.lf_H1.setToolTip(txt)

        # Setup of in_H2
        txt = self.tr(u"""Pole bottom height""")
        self.in_H2.setWhatsThis(txt)
        self.in_H2.setToolTip(txt)

        # Setup of lf_H2
        self.lf_H2.validator().setBottom(0)
        txt = self.tr(u"""Pole bottom height""")
        self.lf_H2.setWhatsThis(txt)
        self.lf_H2.setToolTip(txt)

        # Setup of in_W3
        txt = self.tr(u"""Edge Distance Ploe-coil """)
        self.in_W3.setWhatsThis(txt)
        self.in_W3.setToolTip(txt)

        # Setup of lf_W3
        self.lf_W3.validator().setBottom(0)
        txt = self.tr(u"""Edge Distance Ploe-coil """)
        self.lf_W3.setWhatsThis(txt)
        self.lf_W3.setToolTip(txt)

        # Setup of in_H3
        txt = self.tr(u"""Top Distance Ploe-coil """)
        self.in_H3.setWhatsThis(txt)
        self.in_H3.setToolTip(txt)

        # Setup of lf_H3
        self.lf_H3.validator().setBottom(0)
        txt = self.tr(u"""Top Distance Ploe-coil """)
        self.lf_H3.setWhatsThis(txt)
        self.lf_H3.setToolTip(txt)

        # Setup of in_H4
        txt = self.tr(u"""Bottom Distance Ploe-coil """)
        self.in_H4.setWhatsThis(txt)
        self.in_H4.setToolTip(txt)

        # Setup of lf_H4
        self.lf_H4.validator().setBottom(0)
        txt = self.tr(u"""Bottom Distance Ploe-coil """)
        self.lf_H4.setWhatsThis(txt)
        self.lf_H4.setToolTip(txt)



# -*- coding: utf-8 -*-

from numpy import array


def comp_surface_num(point_list):
    """Compute the surface of the Polygon defined by the points

    Parameters
    ----------
    point_list : list
        List of complex coordinate of the points

    Returns
    -------
    S: float
        Surface of the closed surface [m**2]

    """

    # We have to be sure that the surface is closed
    # (ie point_list[0]=point_list[end]) If it's already the case Xi+1 - Xi =0
    point_list.append(point_list[0])

    point_array = array(point_list)

    # We translate the points so every point have imag >=0
    min_imag = min(point_array.imag)
    if min_imag < 0:
        point_array += abs(min_imag) * 1j

    S_acc = 0
    for ii in range(point_array.size - 1):
        Var_X = point_array[ii + 1].real - point_array[ii].real
        Var_Y = point_array[ii + 1].imag + point_array[ii].imag

        S_acc += Var_Y * Var_X

    return abs(S_acc / 2.0)



# -*- coding: utf-8 -*-

from numpy import arcsin, arctan, cos


def comp_alpha(self):
    """Compute the alpha angle of the slot (cf schematics)

    Parameters
    ----------
    self : HoleM50
        a HoleM50 object

    Returns
    -------
    alpha: float
        Cf schematics [rad]
    """
    Rext = self.get_Rext()

    alpham = 2 * arcsin(self.W0 / (2 * (Rext - self.H1)))

    Harc = (Rext - self.H1) * (1 - cos(alpham / 2))
    return arctan((self.H0 - self.H1 - Harc) / (self.W0 / 2 - self.W1 / 2))



# -*- coding: utf-8 -*-
from ....Methods.Machine.Winding import WindingError


def comp_Ntspc(self, Zs=None):
    """Compute the number of turns in series per phase

    Parameters
    ----------
    self : Winding
        A Winding object
    Zs : int
        Number of slot

    Returns
    -------
    Ntspc: float
        Number of turns in series per phase

    """
    if Zs is None:
        if self.parent is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object."
            )

        if self.parent.slot is None:
            raise WindingError(
                "ERROR: The Winding object must be in a Lamination object with Slot."
            )

        Zs = self.parent.slot.Zs

    # effective number of turns in series per phase from connection matrix
    Ntspc_eff_ = (
        abs(self.comp_connection_mat(Zs).sum(axis=(0, 1))).sum(axis=0) / self.Npcpp / 2
    )
    if Ntspc_eff_.std() != 0:
        self.get_logger().warning(
            "Winding.comp_Ntspc: "
            "Uneven number of turns in series per phase. "
            + "Mean number of turns will be used."
        )

    Ntspc_eff = Ntspc_eff_.mean()

    return Ntspc_eff



# -*- coding: utf-8 -*-
from ....Classes.MeshMat import MeshMat
from ....Classes.PointMat import PointMat
from ....Classes.CellMat import CellMat
from ....Classes.Interpolation import Interpolation
from ....Classes.FPGNSeg import FPGNSeg
from ....Classes.FPGNTri import FPGNTri

from ....Classes.ScalarProductL2 import ScalarProductL2
from ....Classes.RefSegmentP1 import RefSegmentP1
from ....Classes.RefTriangle3 import RefTriangle3


from numpy import array, linspace


def convert(self, meshtype, scale):
    """Convert this object to another type of Mesh object.

    Parameters
    ----------
    self : MeshVTK
        a MeshVTK object
    meshtype : str
        a type of Mesh object
    scale : float
        scale factor

        Returns
    -------
    new_mesh : Mesh
        a Mesh object
    """

    if meshtype == "MeshVTK":
        new_mesh = self.copy()
    elif meshtype == "MeshMat":
        new_mesh = MeshMat(dimension=self.dimension)

        connect_all = self.get_cell()[0]
        point = array(self.get_point())
        nb_pt = point.shape[0]

        new_mesh.point = PointMat(
            coordinate=scale * point, nb_pt=nb_pt, indice=linspace(0, nb_pt - 1, nb_pt)
        )

        min_indice = 0
        for key in connect_all:
            connect = connect_all[key]
            nb_cell = connect.shape[0]
            indices = linspace(min_indice, min_indice + nb_cell - 1, nb_cell, dtype=int)
            min_indice = min_indice + nb_cell

            if key == "line":
                new_mesh.cell["line"] = CellMat(
                    nb_pt_per_cell=2,
                    connectivity=connect,
                    nb_cell=nb_cell,
                    indice=indices,
                )
                interp = Interpolation()
                interp.gauss_point = FPGNSeg()
                interp.ref_cell = RefSegmentP1()
                interp.scalar_product = ScalarProductL2()
                new_mesh.cell["line"].interpolation = interp
            elif key == "line3":
                new_mesh.cell["line3"] = CellMat(
                    nb_pt_per_cell=3,
                    connectivity=connect,
                    nb_cell=nb_cell,
                    indice=indices,
                )
                interp = Interpolation()
                interp.gauss_point = None  # TODO
                interp.ref_cell = None  # TODO
                interp.scalar_product = None  # TODO
                new_mesh.cell["line3"].interpolation = interp
            elif key == "triangle3":
                new_mesh.cell["triangle"] = CellMat(
                    nb_pt_per_cell=3,
                    connectivity=connect,
                    nb_cell=nb_cell,
                    indice=indices,
                )
                interp = Interpolation()
                interp.gauss_point = FPGNTri()
                interp.ref_cell = RefTriangle3()
                interp.scalar_product = ScalarProductL2()
                new_mesh.cell["triangle"].interpolation = interp
            elif key == "quad9":
                new_mesh.cell["quad9"] = CellMat(
                    nb_pt_per_cell=9,
                    connectivity=connect,
                    nb_cell=nb_cell,
                    indice=indices,
                )
                interp = Interpolation()
                interp.gauss_point = None  # TODO
                interp.ref_cell = None  # TODO
                interp.scalar_product = None  # TODO
                new_mesh.cell["quad9"].interpolation = interp

    return new_mesh



# -*- coding: utf-8 -*-

from ....Functions.Geometry.comp_surface_num import comp_surface_num


def comp_surface(self):
    """Compute the SurfRing surface

    Parameters
    ----------
    self : SurfRing
        A SurfRing object

    Returns
    -------
    surf: float
        The SurfRing surface [m**2]

    """

    Sout = self.out_surf.comp_surface()
    Sin = self.in_surf.comp_surface()

    assert Sout > Sin

    return Sout - Sin



from logging import getLogger
from os.path import dirname, isfile
import matplotlib.pyplot as plt
from ezdxf import readfile
from numpy import angle as np_angle
from numpy import array, pi
from PySide2.QtCore import QSize, Qt
from PySide2.QtGui import QIcon, QPixmap
from PySide2.QtWidgets import (
    QComboBox,
    QDialog,
    QFileDialog,
    QMessageBox,
    QPushButton,
    QHeaderView,
)

from ...Classes.HoleUD import HoleUD
from ...Classes.Magnet import Magnet
from ...Classes.SurfLine import SurfLine
from ...GUI.Dxf.dxf_to_pyleecan_list import dxf_to_pyleecan_list
from ...GUI.Resources import pixmap_dict
from ...GUI.Tools.MPLCanvas import MPLCanvas2
from ...GUI.Tools.FloatEdit import FloatEdit
from ...loggers import GUI_LOG_NAME
from .Ui_DXF_Hole import Ui_DXF_Hole

# Column index for table

DEL_COL = 0
HL_COL = 1
TYPE_COL = 2
REF_COL = 3
OFF_COL = 4

ICON_SIZE = 24
# Unselected, selected, selected-bottom-mag
COLOR_LIST = ["k", "r", "c"]


class DXF_Hole(Ui_DXF_Hole, QDialog):
    """Dialog to create HoleUD objects from DXF files"""

    def __init__(self, dxf_path=None, Zh=None, Lmag=None, lam=None):
        """Initialize the Dialog

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        dxf_path : str
            Path to a dxf file to read
        """
        # Widget setup
        QDialog.__init__(self)
        self.setupUi(self)

        # Icon preparation
        self.delete_icon = QPixmap(pixmap_dict["cross"])
        self.delete_icon.scaled(ICON_SIZE, ICON_SIZE, Qt.KeepAspectRatio)
        self.highlight_icon = QPixmap(pixmap_dict["search"])
        self.highlight_icon.scaled(ICON_SIZE, ICON_SIZE, Qt.KeepAspectRatio)

        # Initialize the graph
        self.init_graph()

        # Not used yet
        self.in_coord_center.hide()
        self.lf_center_x.hide()
        self.lf_center_y.hide()
        self.lf_axe_angle.hide()
        self.in_axe_angle.hide()

        # Set default values
        if Zh is not None:
            self.si_Zh.setValue(Zh)
        if Lmag is not None:
            self.lf_mag_len.setValue(Lmag)
        if lam is None:
            self.lam = lam
        else:
            self.lam = lam.copy()

        # Init properties
        self.line_list = list()  # List of line from DXF
        self.selected_list = list()  # List of currently selected lines
        self.surf_list = list()  # List of defined surfaces
        self.lf_center_x.setValue(0)
        self.lf_center_y.setValue(0)
        self.lf_scaling.validator().setBottom(0)
        self.lf_scaling.setValue(1)

        # Load the DXF file if provided
        self.dxf_path = dxf_path
        if dxf_path is not None and isfile(dxf_path):
            self.open_document()

        # Setup Path selector for DXF files
        self.w_path_selector.obj = self
        self.w_path_selector.param_name = "dxf_path"
        self.w_path_selector.verbose_name = "DXF File"
        self.w_path_selector.extension = "DXF file (*.dxf)"
        self.w_path_selector.set_path_txt(self.dxf_path)
        self.w_path_selector.update()

        # Set table column width
        header = self.w_surface_list.horizontalHeader()
        header.setSectionResizeMode(DEL_COL, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(HL_COL, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(TYPE_COL, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(REF_COL, QHeaderView.ResizeToContents)
        header.setSectionResizeMode(OFF_COL, QHeaderView.ResizeToContents)

        # Connect signals to slot
        self.w_path_selector.pathChanged.connect(self.open_document)
        self.b_save.pressed.connect(self.save)
        self.b_plot.pressed.connect(self.plot)
        self.b_reset.pressed.connect(self.update_graph)
        self.b_cancel.pressed.connect(self.remove_selection)

        # Display the GUI
        self.show()

    def open_document(self):
        """Open a new dxf in the viewer

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """

        getLogger(GUI_LOG_NAME).debug("Reading dxf file: " + self.dxf_path)
        # Read the DXF file
        try:
            document = readfile(self.dxf_path)
            modelspace = document.modelspace()
            # Convert DXF to pyleecan objects
            self.line_list = dxf_to_pyleecan_list(modelspace)
            # Display
            # selected line: 0: unselected, 1:selected, 2: selected bottom magnet
            self.selected_list = [0 for line in self.line_list]
            self.update_graph()
        except Exception as e:
            QMessageBox().critical(
                self,
                self.tr("Error"),
                self.tr("Error while reading dxf file:\n" + str(e)),
            )

    def init_graph(self):
        """Initialize the viewer

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """
        fig, axes = self.w_viewer.fig, self.w_viewer.axes
        axes.set_axis_off()

        # Setup interaction with graph
        def select_line(event):
            """Function to select/unselect the closest line from click"""
            X = event.xdata  # X position of the click
            Y = event.ydata  # Y position of the click
            # Get closer pyleecan object
            Z = X + 1j * Y
            min_dist = float("inf")
            closest_id = -1
            for ii, line in enumerate(self.line_list):
                line_dist = line.comp_distance(Z)
                if line_dist < min_dist:
                    closest_id = ii
                    min_dist = line_dist
            # Select/unselect line
            if self.selected_list[closest_id] == 0:  # Unselected to selected
                self.selected_list[closest_id] = 1
            elif self.selected_list[closest_id] == 1:  # Selected to selected bottom mag
                if 2 in self.selected_list:
                    current_bot_mag = self.selected_list.index(2)
                    # Only one selected bottom mag line at the time
                    point_list = array(self.line_list[current_bot_mag].discretize(20))
                    self.selected_list[current_bot_mag] = 1
                    axes.plot(point_list.real, point_list.imag, COLOR_LIST[1], zorder=2)
                self.selected_list[closest_id] = 2
            elif self.selected_list[closest_id] == 2:
                # selected bottom mag to Unselected
                self.selected_list[closest_id] = 0
            # Change line color
            point_list = array(self.line_list[closest_id].discretize(20))
            color = COLOR_LIST[self.selected_list[closest_id]]
            axes.plot(point_list.real, point_list.imag, color, zorder=2)
            self.w_viewer.draw()

            # Check if the surface is complete
            if self.check_selection():
                self.add_surface()

        def zoom(event):
            """Function to zoom/unzoom according the mouse wheel"""

            base_scale = 0.3  # Scaling factor
            # get the current x and y limits
            ax = self.w_viewer.axes
            cur_xlim = ax.get_xlim()
            cur_ylim = ax.get_ylim()
            cur_xrange = (cur_xlim[1] - cur_xlim[0]) * 0.5
            cur_yrange = (cur_ylim[1] - cur_ylim[0]) * 0.5
            xdata = event.xdata  # get event x location
            ydata = event.ydata  # get event y location
            if event.button == "down":
                # deal with zoom in
                scale_factor = 1 / base_scale
            elif event.button == "up":
                # deal with zoom out
                scale_factor = base_scale
            else:
                # deal with something that should never happen
                scale_factor = 1
            # set new limits
            ax.set_xlim(
                [xdata - cur_xrange * scale_factor, xdata + cur_xrange * scale_factor]
            )
            ax.set_ylim(
                [ydata - cur_yrange * scale_factor, ydata + cur_yrange * scale_factor]
            )
            self.w_viewer.draw()  # force re-draw

        # Connect the function
        self.w_viewer.mpl_connect("button_press_event", select_line)
        self.w_viewer.mpl_connect("scroll_event", zoom)

        # Axis cleanup
        axes.axis("equal")
        axes.set_axis_off()

    def update_graph(self):
        """Clean and redraw all the lines in viewer

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """
        fig, axes = self.w_viewer.fig, self.w_viewer.axes
        axes.clear()
        axes.set_axis_off()

        # Draw the lines in the correct color
        for ii, line in enumerate(self.line_list):
            point_list = array(line.discretize(20))
            color = COLOR_LIST[self.selected_list[ii]]
            axes.plot(point_list.real, point_list.imag, color, zorder=1)

        self.w_viewer.draw()

    def check_selection(self):
        """Check if every line in the selection form a surface

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object

        Returns
        -------
        is_surf : bool
            True if it forms a surface
        """

        # Create list of begin and end point for all lines
        point_list = list()
        for ii, line in enumerate(self.line_list):
            if self.selected_list[ii]:
                point_list.append(line.get_begin())
                point_list.append(line.get_end())

        # Check with a tolerance if every point is twice in the list
        if len(point_list) == 0:
            return False

        for p1 in point_list:
            count = 0
            for p2 in point_list:
                if abs(p1 - p2) < 1e-9:
                    count += 1
            if count != 2:
                return False

        return True

    def add_surface(self):
        """Validate the selection and create a surface object

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """

        # Get all the selected lines
        line_list = list()
        index_list = list()
        for ii, line in enumerate(self.line_list):
            if self.selected_list[ii]:
                index_list.append(str(ii))
                line_list.append(line.copy())
        # Sort the lines (begin = end)
        curve_list = list()
        curve_list.append(line_list.pop())
        while len(line_list) > 0:
            end = curve_list[-1].get_end()
            for ii in range(len(line_list)):
                if abs(line_list[ii].get_begin() - end) < 1e-9:
                    break
                if abs(line_list[ii].get_end() - end) < 1e-9:
                    line_list[ii].reverse()
                    break
            curve_list.append(line_list.pop(ii))
        # Create the Surface object
        self.surf_list.append(SurfLine(line_list=curve_list))
        self.surf_list[-1].comp_point_ref(is_set=True)

        # Add a line in the Table
        nrows = self.w_surface_list.rowCount()
        self.w_surface_list.setRowCount(nrows + 1)
        # Adding Surface Type combobox
        combobox = QComboBox()
        combobox.addItems(["Hole", "Magnet"])
        self.w_surface_list.setCellWidget(
            nrows,
            TYPE_COL,
            combobox,
        )
        if 2 in self.selected_list:
            combobox.setCurrentIndex(1)  # Magnet
        combobox.currentIndexChanged.connect(self.enable_magnetization)

        # Adding Delete button
        del_button = QPushButton("")
        del_button.setIcon(QIcon(self.delete_icon))
        del_button.pressed.connect(self.delete_surface)
        self.w_surface_list.setCellWidget(
            nrows,
            DEL_COL,
            del_button,
        )

        # Adding Highlight button
        HL_button = QPushButton("")
        HL_button.setIcon(QIcon(self.highlight_icon))
        HL_button.pressed.connect(self.highlight_surface)
        self.w_surface_list.setCellWidget(
            nrows,
            HL_COL,
            HL_button,
        )

        # Add reference combobox
        combobox = QComboBox()
        combobox.addItems(index_list)
        self.w_surface_list.setCellWidget(
            nrows,
            REF_COL,
            combobox,
        )
        if 2 in self.selected_list:
            combobox.setCurrentIndex(
                index_list.index(str(self.selected_list.index(2)))
            )  #
        else:
            combobox.setEnabled(False)

        # Add Offset FloatEdit
        lf_off = FloatEdit()
        lf_off.validator().setBottom(-360)
        lf_off.validator().setTop(360)
        lf_off.setValue(0)
        # lf_off.setText("0")
        lf_off.setEnabled(2 in self.selected_list)
        self.w_surface_list.setCellWidget(
            nrows,
            OFF_COL,
            lf_off,
        )

        # Remove selection to start new one
        self.remove_selection()

    def enable_magnetization(self):
        """Enable/Disable the combobox/float edit for magnetization according to type"""
        for ii in range(self.w_surface_list.rowCount()):
            if self.w_surface_list.cellWidget(ii, TYPE_COL).currentIndex() == 0:
                self.w_surface_list.cellWidget(ii, REF_COL).setEnabled(False)
                self.w_surface_list.cellWidget(ii, OFF_COL).setEnabled(False)
            else:
                self.w_surface_list.cellWidget(ii, REF_COL).setEnabled(True)
                self.w_surface_list.cellWidget(ii, OFF_COL).setEnabled(True)

    def remove_selection(self):
        # Remove selection
        self.selected_list = [0 for line in self.line_list]
        self.update_graph()

    def get_hole(self):
        """Generate the HoleUD object corresponding to the selected surfaces

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object

        Returns
        -------
        hole : HoleUD
            User defined hole according to selected surfaces
        """

        if self.lf_scaling.value() == 0:  # Avoid error
            self.lf_scaling.setValue(1)
        hole = HoleUD(surf_list=[])
        bottom_list = list()
        offset_list = list()
        # Set labels
        Nmag = 0
        for ii in range(self.w_surface_list.rowCount()):
            hole.surf_list.append(self.surf_list[ii].copy())
            hole.surf_list[ii].scale(self.lf_scaling.value())
            if self.w_surface_list.cellWidget(ii, TYPE_COL).currentIndex() == 0:
                hole.surf_list[ii].label = "Hole"
            else:
                hole.surf_list[ii].label = "HoleMagnet"
                Nmag += 1
            bottom_list.append(
                self.line_list[
                    int(self.w_surface_list.cellWidget(ii, REF_COL).currentText())
                ]
            )
            offset_list.append(self.w_surface_list.cellWidget(ii, OFF_COL).value())
        # Create magnet objects
        hole.magnet_dict = dict()
        for ii in range(Nmag):
            hole.magnet_dict["magnet_" + str(ii)] = Magnet(type_magnetization=1)

        # Sort the surfaces
        angles = [np_angle(surf.point_ref) for surf in hole.surf_list]
        idx = sorted(range(len(angles)), key=lambda k: angles[k])
        surf_list_sorted = [hole.surf_list[ii] for ii in idx]
        bottom_list_sorted = [bottom_list[ii] for ii in idx]
        offset_list_sorted = [offset_list[ii] for ii in idx]
        hole.surf_list = surf_list_sorted

        # Rotation
        Zref = sum([surf.point_ref for surf in hole.surf_list])
        for surf in hole.surf_list:
            surf.rotate(-1 * np_angle(Zref))

        # Magnetization dict
        mag_dict = dict()
        Nmag = 0
        for ii in range(len(hole.surf_list)):
            if "Magnet" in hole.surf_list[ii].label:
                line = bottom_list_sorted[ii].copy()
                line.rotate(-1 * np_angle(Zref))
                mag_dict["magnet_" + str(Nmag)] = line.comp_normal()
                mag_dict["magnet_" + str(Nmag)] += offset_list_sorted[ii] * pi / 180
                Nmag += 1
        hole.magnetization_dict_offset = mag_dict

        # Set metadata
        hole.Zh = self.si_Zh.value()
        for magnet in hole.magnet_dict.values():
            magnet.Lmag = self.lf_mag_len.value()

        # Remove all materials => To be set in GUI
        hole.mat_void = None
        for magnet in hole.magnet_dict.values():
            magnet.mat_type = None

        return hole

    def plot(self):
        """Plot the current state of the hole

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """
        hole = self.get_hole()
        if self.lam is None:
            hole.plot(is_add_arrow=True)
        else:
            fig, (ax1, ax2) = plt.subplots(1, 2)
            hole.plot(fig=fig, ax=ax1, is_add_arrow=True)
            self.lam.hole = [hole]
            self.lam.plot(fig=fig, ax=ax2)

    def delete_surface(self):
        """Delete a selected surface

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """
        nrow = self.w_surface_list.currentRow()
        self.surf_list.pop(nrow)
        self.w_surface_list.removeRow(nrow)

    def highlight_surface(self):
        """Highlight a surface to find it on the viewer

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """
        self.selected_list = [0 for line in self.line_list]
        surf = self.surf_list[self.w_surface_list.currentRow()]
        # Find the index of the surface line in self.line_list
        for surf_line in surf.line_list:
            mid = surf_line.get_middle()
            for ii, line in enumerate(self.line_list):
                if abs(mid - line.get_middle()) < 1e-6:
                    self.selected_list[ii] = 1
                    self.w_viewer.axes.text(
                        mid.real,
                        mid.imag,
                        str(ii),
                        # fontsize=fontsize,
                    )
        self.update_graph()
        # Add Label
        for ii in range(len(self.selected_list)):
            if self.selected_list[ii] == 1:
                Zmid = self.line_list[ii].get_middle()
                self.w_viewer.axes.text(
                    Zmid.real,
                    Zmid.imag,
                    str(ii),
                    # fontsize=fontsize,
                )

    def save(self):
        """Save the HoleUD object in a json file

        Parameters
        ----------
        self : DXF_Hole
            a DXF_Hole object
        """

        hole = self.get_hole()

        save_file_path = QFileDialog.getSaveFileName(
            self, self.tr("Save file"), dirname(self.dxf_path), "Json (*.json)"
        )[0]
        if save_file_path not in ["", ".json", None]:
            self.save_path = save_file_path
            hole.save(save_file_path)
            self.accept()



class PointArc2Error(Exception):
    """ """

    pass


class AngleArc2Error(Exception):
    """ """

    pass


class NbPointArc2DError(Exception):
    """ """

    pass


class AngleRotationArc2Error(Exception):
    """ """

    pass


class PointTranslateArc2Error(Exception):
    """ """

    pass



# -*- coding: utf-8 -*-
# File generated according to Generator/ClassesRef/Slot/SlotUD.csv
# WARNING! All changes made in this file will be lost!
"""Method code available at https://github.com/Eomys/pyleecan/tree/master/pyleecan/Methods/Slot/SlotUD
"""

from os import linesep
from sys import getsizeof
from logging import getLogger
from ._check import check_var, raise_
from ..Functions.get_logger import get_logger
from ..Functions.save import save
from ..Functions.copy import copy
from ..Functions.load import load_init_dict
from ..Functions.Load.import_class import import_class
from .Slot import Slot

# Import all class method
# Try/catch to remove unnecessary dependencies in unused method
try:
    from ..Methods.Slot.SlotUD.build_geometry import build_geometry
except ImportError as error:
    build_geometry = error

try:
    from ..Methods.Slot.SlotUD.set_from_point_list import set_from_point_list
except ImportError as error:
    set_from_point_list = error

try:
    from ..Methods.Slot.SlotUD.get_surface_active import get_surface_active
except ImportError as error:
    get_surface_active = error

try:
    from ..Methods.Slot.SlotUD.check import check
except ImportError as error:
    check = error


from ._check import InitUnKnowClassError
from .Line import Line


class SlotUD(Slot):
    """"User defined" Slot from a line list. """

    VERSION = 1

    # Check ImportError to remove unnecessary dependencies in unused method
    # cf Methods.Slot.SlotUD.build_geometry
    if isinstance(build_geometry, ImportError):
        build_geometry = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotUD method build_geometry: " + str(build_geometry)
                )
            )
        )
    else:
        build_geometry = build_geometry
    # cf Methods.Slot.SlotUD.set_from_point_list
    if isinstance(set_from_point_list, ImportError):
        set_from_point_list = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotUD method set_from_point_list: "
                    + str(set_from_point_list)
                )
            )
        )
    else:
        set_from_point_list = set_from_point_list
    # cf Methods.Slot.SlotUD.get_surface_active
    if isinstance(get_surface_active, ImportError):
        get_surface_active = property(
            fget=lambda x: raise_(
                ImportError(
                    "Can't use SlotUD method get_surface_active: "
                    + str(get_surface_active)
                )
            )
        )
    else:
        get_surface_active = get_surface_active
    # cf Methods.Slot.SlotUD.check
    if isinstance(check, ImportError):
        check = property(
            fget=lambda x: raise_(
                ImportError("Can't use SlotUD method check: " + str(check))
            )
        )
    else:
        check = check
    # save and copy methods are available in all object
    save = save
    copy = copy
    # get_logger method is available in all object
    get_logger = get_logger

    def __init__(
        self,
        line_list=-1,
        wind_begin_index=None,
        wind_end_index=None,
        type_line_wind=0,
        Zs=36,
        init_dict=None,
        init_str=None,
    ):
        """Constructor of the class. Can be use in three ways :
        - __init__ (arg1 = 1, arg3 = 5) every parameters have name and default values
            for pyleecan type, -1 will call the default constructor
        - __init__ (init_dict = d) d must be a dictionnary with property names as keys
        - __init__ (init_str = s) s must be a string
        s is the file path to load

        ndarray or list can be given for Vector and Matrix
        object or dict can be given for pyleecan Object"""

        if init_str is not None:  # Load from a file
            init_dict = load_init_dict(init_str)[1]
        if init_dict is not None:  # Initialisation by dict
            assert type(init_dict) is dict
            # Overwrite default value with init_dict content
            if "line_list" in list(init_dict.keys()):
                line_list = init_dict["line_list"]
            if "wind_begin_index" in list(init_dict.keys()):
                wind_begin_index = init_dict["wind_begin_index"]
            if "wind_end_index" in list(init_dict.keys()):
                wind_end_index = init_dict["wind_end_index"]
            if "type_line_wind" in list(init_dict.keys()):
                type_line_wind = init_dict["type_line_wind"]
            if "Zs" in list(init_dict.keys()):
                Zs = init_dict["Zs"]
        # Set the properties (value check and convertion are done in setter)
        self.line_list = line_list
        self.wind_begin_index = wind_begin_index
        self.wind_end_index = wind_end_index
        self.type_line_wind = type_line_wind
        # Call Slot init
        super(SlotUD, self).__init__(Zs=Zs)
        # The class is frozen (in Slot init), for now it's impossible to
        # add new properties

    def __str__(self):
        """Convert this object in a readeable string (for print)"""

        SlotUD_str = ""
        # Get the properties inherited from Slot
        SlotUD_str += super(SlotUD, self).__str__()
        if len(self.line_list) == 0:
            SlotUD_str += "line_list = []" + linesep
        for ii in range(len(self.line_list)):
            tmp = (
                self.line_list[ii].__str__().replace(linesep, linesep + "\t") + linesep
            )
            SlotUD_str += "line_list[" + str(ii) + "] =" + tmp + linesep + linesep
        SlotUD_str += "wind_begin_index = " + str(self.wind_begin_index) + linesep
        SlotUD_str += "wind_end_index = " + str(self.wind_end_index) + linesep
        SlotUD_str += "type_line_wind = " + str(self.type_line_wind) + linesep
        return SlotUD_str

    def __eq__(self, other):
        """Compare two objects (skip parent)"""

        if type(other) != type(self):
            return False

        # Check the properties inherited from Slot
        if not super(SlotUD, self).__eq__(other):
            return False
        if other.line_list != self.line_list:
            return False
        if other.wind_begin_index != self.wind_begin_index:
            return False
        if other.wind_end_index != self.wind_end_index:
            return False
        if other.type_line_wind != self.type_line_wind:
            return False
        return True

    def compare(self, other, name="self"):
        """Compare two objects and return list of differences"""

        if type(other) != type(self):
            return ["type(" + name + ")"]
        diff_list = list()

        # Check the properties inherited from Slot
        diff_list.extend(super(SlotUD, self).compare(other, name=name))
        if (other.line_list is None and self.line_list is not None) or (
            other.line_list is not None and self.line_list is None
        ):
            diff_list.append(name + ".line_list None mismatch")
        elif self.line_list is None:
            pass
        elif len(other.line_list) != len(self.line_list):
            diff_list.append("len(" + name + ".line_list)")
        else:
            for ii in range(len(other.line_list)):
                diff_list.extend(
                    self.line_list[ii].compare(
                        other.line_list[ii], name=name + ".line_list[" + str(ii) + "]"
                    )
                )
        if other._wind_begin_index != self._wind_begin_index:
            diff_list.append(name + ".wind_begin_index")
        if other._wind_end_index != self._wind_end_index:
            diff_list.append(name + ".wind_end_index")
        if other._type_line_wind != self._type_line_wind:
            diff_list.append(name + ".type_line_wind")
        return diff_list

    def __sizeof__(self):
        """Return the size in memory of the object (including all subobject)"""

        S = 0  # Full size of the object

        # Get size of the properties inherited from Slot
        S += super(SlotUD, self).__sizeof__()
        if self.line_list is not None:
            for value in self.line_list:
                S += getsizeof(value)
        S += getsizeof(self.wind_begin_index)
        S += getsizeof(self.wind_end_index)
        S += getsizeof(self.type_line_wind)
        return S

    def as_dict(self, **kwargs):
        """
        Convert this object in a json serializable dict (can be use in __init__).
        Optional keyword input parameter is for internal use only
        and may prevent json serializability.
        """

        # Get the properties inherited from Slot
        SlotUD_dict = super(SlotUD, self).as_dict(**kwargs)
        if self.line_list is None:
            SlotUD_dict["line_list"] = None
        else:
            SlotUD_dict["line_list"] = list()
            for obj in self.line_list:
                if obj is not None:
                    SlotUD_dict["line_list"].append(obj.as_dict(**kwargs))
                else:
                    SlotUD_dict["line_list"].append(None)
        SlotUD_dict["wind_begin_index"] = self.wind_begin_index
        SlotUD_dict["wind_end_index"] = self.wind_end_index
        SlotUD_dict["type_line_wind"] = self.type_line_wind
        # The class name is added to the dict for deserialisation purpose
        # Overwrite the mother class name
        SlotUD_dict["__class__"] = "SlotUD"
        return SlotUD_dict

    def _set_None(self):
        """Set all the properties to None (except pyleecan object)"""

        self.line_list = None
        self.wind_begin_index = None
        self.wind_end_index = None
        self.type_line_wind = None
        # Set to None the properties inherited from Slot
        super(SlotUD, self)._set_None()

    def _get_line_list(self):
        """getter of line_list"""
        if self._line_list is not None:
            for obj in self._line_list:
                if obj is not None:
                    obj.parent = self
        return self._line_list

    def _set_line_list(self, value):
        """setter of line_list"""
        if type(value) is list:
            for ii, obj in enumerate(value):
                if type(obj) is dict:
                    class_obj = import_class(
                        "pyleecan.Classes", obj.get("__class__"), "line_list"
                    )
                    value[ii] = class_obj(init_dict=obj)
                if value[ii] is not None:
                    value[ii].parent = self
        if value == -1:
            value = list()
        check_var("line_list", value, "[Line]")
        self._line_list = value

    line_list = property(
        fget=_get_line_list,
        fset=_set_line_list,
        doc=u"""list of line to draw the edges of the slot

        :Type: [Line]
        """,
    )

    def _get_wind_begin_index(self):
        """getter of wind_begin_index"""
        return self._wind_begin_index

    def _set_wind_begin_index(self, value):
        """setter of wind_begin_index"""
        check_var("wind_begin_index", value, "int")
        self._wind_begin_index = value

    wind_begin_index = property(
        fget=_get_wind_begin_index,
        fset=_set_wind_begin_index,
        doc=u"""Index of the first line to include in the winding

        :Type: int
        """,
    )

    def _get_wind_end_index(self):
        """getter of wind_end_index"""
        return self._wind_end_index

    def _set_wind_end_index(self, value):
        """setter of wind_end_index"""
        check_var("wind_end_index", value, "int")
        self._wind_end_index = value

    wind_end_index = property(
        fget=_get_wind_end_index,
        fset=_set_wind_end_index,
        doc=u"""Index of the last line to include in the winding

        :Type: int
        """,
    )

    def _get_type_line_wind(self):
        """getter of type_line_wind"""
        return self._type_line_wind

    def _set_type_line_wind(self, value):
        """setter of type_line_wind"""
        check_var("type_line_wind", value, "int", Vmin=0, Vmax=1)
        self._type_line_wind = value

    type_line_wind = property(
        fget=_get_type_line_wind,
        fset=_set_type_line_wind,
        doc=u"""0 to close winding with Segment, 1 for Arc1

        :Type: int
        :min: 0
        :max: 1
        """,
    )



# -*- coding: utf-8 -*-

from numpy import pi


def comp_surface_active(self):
    """Compute the Slot inner surface for winding (by analytical computation)

    Parameters
    ----------
    self : SlotW22
        A SlotW22 object

    Returns
    -------
    Swind: float
        Slot inner surface for winding [m**2]

    """
    Rbo = self.get_Rbo()

    if self.is_outwards():
        # Surface of the external disk
        Sext = ((self.H2 + self.H0 + Rbo) ** 2) * pi

        # Surface of the internal disk
        Sint = ((self.H0 + Rbo) ** 2) * pi
    else:
        # Surface of the external disk
        Sext = ((Rbo - self.H0) ** 2) * pi

        # Surface of the internal disk
        Sint = ((Rbo - self.H2 - self.H0) ** 2) * pi

    # Surface of the ring
    Sring = Sext - Sint

    # Only an W2 angle of the ring
    return Sring * self.W2 / (2 * pi)
